@@ -1,4 +1,6 @@
+#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <CL/opencl.h>
 #include <poclu.h>
 
@@ -21,7 +23,8 @@ exec_dot_product_kernel(const char *program_source, size_t source_size,
   cl_context  context; 
   cl_command_queue cmd_queue; 
   cl_device_id  *devices; 
-  cl_program  program; 
+  cl_platform_id platform;
+  cl_program  program;
   cl_kernel  kernel; 
   cl_mem       memobjs[3]; 
   size_t       global_work_size[1]; 
@@ -29,6 +32,9 @@ exec_dot_product_kernel(const char *program_source, size_t source_size,
   size_t       cb; 
   cl_int       err; 
   int          i;
+
+  clGetPlatformIDs (1, &platform, NULL);
+
   context = poclu_create_any_context();
   if (context == (cl_context)0) 
     return -1; 
@@ -36,8 +42,18 @@ exec_dot_product_kernel(const char *program_source, size_t source_size,
   // get the list of GPU devices associated with context 
   clGetContextInfo(context, CL_CONTEXT_DEVICES, 0, NULL, &cb); 
   devices = (cl_device_id *) malloc(cb);
-  clGetContextInfo(context, CL_CONTEXT_DEVICES, cb, devices, NULL); 
- 
+  clGetContextInfo(context, CL_CONTEXT_DEVICES, cb, devices, NULL);
+
+  char extensions[1024];
+  err = clGetDeviceInfo (devices[0], CL_DEVICE_EXTENSIONS, 1024, extensions,
+                         NULL);
+  CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo");
+  if (strstr (extensions, "cl_khr_spir") == NULL)
+    {
+      printf ("SPIR not supported, cannot run the test\n");
+      return -1;
+    }
+
   // create a command-queue 
   cmd_queue = clCreateCommandQueue(context, devices[0], 0, NULL); 
   if (cmd_queue == (cl_command_queue)0) 
@@ -101,7 +117,7 @@ exec_dot_product_kernel(const char *program_source, size_t source_size,
     } 
  
   // build the program 
-  err = clBuildProgram(program, 0, NULL, NULL, NULL, NULL); 
+  err = clBuildProgram(program, 0, NULL, "-x spir -spir-std=1.2", NULL, NULL);
   if (err != CL_SUCCESS) 
     { 
       delete_memobjs(memobjs, 3); 
@@ -184,7 +200,8 @@ exec_dot_product_kernel(const char *program_source, size_t source_size,
   delete_memobjs(memobjs, 3); 
   clReleaseKernel(kernel); 
   clReleaseProgram(program); 
-  clReleaseCommandQueue(cmd_queue); 
+  clUnloadPlatformCompiler (platform);
+  clReleaseCommandQueue(cmd_queue);
   clReleaseContext(context); 
   return 0; // success... 
 }
