@@ -55,41 +55,46 @@
 # undef LLVM_5_0
 # define LLVM_5_0
 
+#elif (__clang_major__ == 6)
+
+# undef LLVM_6_0
+# define LLVM_6_0
+
 #else
 
 #error Unsupported Clang/LLVM version.
 
 #endif
 
-#if (defined LLVM_3_6)
-# define LLVM_OLDER_THAN_3_7 1
-# define LLVM_OLDER_THAN_3_8 1
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
+#ifndef LLVM_6_0
+#define LLVM_OLDER_THAN_6_0 1
+
+#ifndef LLVM_5_0
+#define LLVM_OLDER_THAN_5_0 1
+
+#ifndef LLVM_4_0
+#define LLVM_OLDER_THAN_4_0 1
+
+#ifndef LLVM_3_9
+#define LLVM_OLDER_THAN_3_9 1
+
+#ifndef LLVM_3_8
+#define LLVM_OLDER_THAN_3_8 1
+
+#ifndef LLVM_3_7
+#define LLVM_OLDER_THAN_3_7 1
+
+#ifndef LLVM_3_6
+#define LLVM_OLDER_THAN_3_6 1
+
+#endif
+#endif
+#endif
+#endif
+#endif
+#endif
 #endif
 
-#if (defined LLVM_3_7)
-# define LLVM_OLDER_THAN_3_8 1
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
-
-#if (defined LLVM_3_8)
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
-
-#if (defined LLVM_3_9)
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
-
-#if (defined LLVM_4_0)
-# define LLVM_OLDER_THAN_5_0 1
-#endif
 
 #include "_kernel_constants.h"
 
