@@ -50,6 +50,12 @@ struct _mem_mapping {
   size_t offset; /* offset to the beginning of the buffer */
   size_t size;
   mem_mapping_t *prev, *next;
+  /* This is required, because two clEnqueueMap() with the same buffer+size+offset,
+     will create two identical mappings in the buffer->mappings LL.
+     Without this flag, both corresponding clEnqUnmap()s will find
+     the same mapping (the first one in mappings LL), which will lead
+     to memory double-free corruption later. */
+  long unmap_requested;
 };
 
 typedef struct _mem_destructor_callback mem_destructor_callback_t;
@@ -303,32 +309,35 @@ struct _cl_command_node
   volatile cl_int ready;
 };
 
-/* Additional LLVM version macros to simplify ifdefs */
-#if (defined LLVM_3_6)
-# define LLVM_OLDER_THAN_3_7 1
-# define LLVM_OLDER_THAN_3_8 1
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
+#ifndef LLVM_6_0
+#define LLVM_OLDER_THAN_6_0 1
+
+#ifndef LLVM_5_0
+#define LLVM_OLDER_THAN_5_0 1
+
+#ifndef LLVM_4_0
+#define LLVM_OLDER_THAN_4_0 1
+
+#ifndef LLVM_3_9
+#define LLVM_OLDER_THAN_3_9 1
+
+#ifndef LLVM_3_8
+#define LLVM_OLDER_THAN_3_8 1
+
+#ifndef LLVM_3_7
+#define LLVM_OLDER_THAN_3_7 1
+
+#ifndef LLVM_3_6
+#define LLVM_OLDER_THAN_3_6 1
+
+#endif
+#endif
+#endif
+#endif
+#endif
+#endif
 #endif
 
-#if (defined LLVM_3_7)
-# define LLVM_OLDER_THAN_3_8 1
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
-
-#if (defined LLVM_3_8)
-# define LLVM_OLDER_THAN_3_9 1
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
-
-#if (defined LLVM_3_9)
-# define LLVM_OLDER_THAN_4_0 1
-# define LLVM_OLDER_THAN_5_0 1
-#endif
 
 #if (defined LLVM_4_0)
 # define LLVM_OLDER_THAN_5_0 1
