@@ -4,6 +4,25 @@
 #  error "cl_khr_fp64 requires cl_khr_int64"
 #endif
 
+#ifdef __CBUILD__
+
+/* Define the required OpenCL C fixed size types in a C compatible way.
+   The C-based kernel library implementations need these. Avoid using
+   stdint.h types as it might not be available for Clang during
+   kernel compilation. */
+typedef unsigned char uchar;
+typedef unsigned short ushort;
+typedef unsigned int uint;
+
+#ifdef cl_khr_int64
+typedef unsigned long ulong;
+#else
+typedef unsigned int ulong;
+#endif
+
+typedef ulong size_t;
+
+#endif
 
 
 /* Disable undefined datatypes */
@@ -20,17 +39,19 @@ typedef struct error_undefined_type_ulong error_undefined_type_ulong;
 #  define ulong error_undefined_type_ulong
 #endif
 
+#ifdef __CBUILD__
 #ifndef cl_khr_fp16
-typedef struct error_undefined_type_half error_undefined_type_half;
-#  define half error_undefined_type_half
+typedef short half;
 #endif
+#endif
+
 
 #ifndef cl_khr_fp64
 typedef struct error_undefined_type_double error_undefined_type_double;
 #  define double error_undefined_type_double
 #endif
 
-
+#ifndef __CBUILD__
 
 /* Define unsigned datatypes */
 
@@ -48,26 +69,4 @@ typedef __PTRDIFF_TYPE__ ptrdiff_t;
 typedef ptrdiff_t intptr_t;
 typedef size_t uintptr_t;
 
-
-/* Image types.
- * Note: there is a duplicate definition in
- * lib/CL/devices/dev_image.h - keep in sync?
- */
-typedef int dev_sampler_t;
-
-typedef struct dev_image_t {
-  void* data;
-  int width;
-  int height;
-  int depth;
-  int image_array_size;
-  int row_pitch;
-  int slice_pitch;
-  int num_mip_levels; /* maybe not needed */
-  int num_samples; /* maybe not needed */
-  int order;
-  int data_type;
-  int num_channels;
-  int elem_size;
-} dev_image_t;
-
+#endif
