@@ -99,6 +99,9 @@ poclu_cl_half_to_float(cl_half value);
 POCLU_API char * POCLU_CALL
 poclu_read_file(char* filemane);
 
+POCLU_API char * POCLU_CALL
+poclu_read_binfile(char *filename, size_t *len);
+
 POCLU_API int POCLU_CALL
 poclu_write_file(char* filemane, char* content, size_t size);
 
@@ -109,6 +112,26 @@ poclu_write_file(char* filemane, char* content, size_t size);
 POCLU_API int POCLU_CALL
 check_cl_error(cl_int cl_err, int line, const char* func_name);
 
+#define _POCLU_CHECK_CL_ERROR_INNER(cond, func, line)                   \
+do {                                                                    \
+   if(check_cl_error(cond, line, func))                                 \
+     exit(EXIT_FAILURE);                                                \
+} while (0)
+
+#define CHECK_CL_ERROR(cond) _POCLU_CHECK_CL_ERROR_INNER(cond, __PRETTY_FUNCTION__, __LINE__)
+
+#define CHECK_OPENCL_ERROR_IN(message) _POCLU_CHECK_CL_ERROR_INNER(err, message, __LINE__)
+
+
+#define TEST_ASSERT(EXP)                                                \
+do {                                                                    \
+  if (!(EXP)) {                                                         \
+    fprintf(stderr, "Assertion: \n" #EXP "\nfailed on %s:%i\n",         \
+        __FILE__, __LINE__);                                            \
+    return EXIT_FAILURE;                                                \
+  }                                                                     \
+} while (0)
+
 #ifdef __cplusplus
 }
 #endif
