@@ -23,7 +23,6 @@
 
 #include "pocl_cl.h"
 #include "pocl_util.h"
-#include "pocl_queue_util.h"
 
 CL_API_ENTRY cl_command_queue CL_API_CALL
 POname(clCreateCommandQueue)(cl_context context, 
@@ -35,18 +34,22 @@ POname(clCreateCommandQueue)(cl_context context,
   int errcode;
   cl_bool found = CL_FALSE;
 
+  POCL_GOTO_ERROR_COND ((context == NULL), CL_INVALID_CONTEXT);
+
+  POCL_GOTO_ERROR_COND ((device == NULL), CL_INVALID_DEVICE);
+
   POCL_MSG_PRINT_INFO("Create Command queue on device %d\n", device->dev_id);
 
   /* validate flags */
   POCL_GOTO_ERROR_ON((properties > (1<<2)-1), CL_INVALID_VALUE,
             "Properties must be <= 3 (there are only 2)\n");
 
-  if (pocl_debug_messages)
+  if (POCL_DEBUGGING_ON)
     properties |= CL_QUEUE_PROFILING_ENABLE;
 
   for (i=0; i<context->num_devices; i++)
     {
-      if (context->devices[i] == POCL_REAL_DEV(device))
+      if (context->devices[i] == pocl_real_dev (device))
         found = CL_TRUE;
     }
 
@@ -67,7 +70,6 @@ POname(clCreateCommandQueue)(cl_context context,
   command_queue->properties = properties;
   command_queue->barrier = NULL;
   command_queue->events = NULL;
-  command_queue->root = NULL;
   command_queue->command_count = 0;
   command_queue->last_event.event = NULL;
   command_queue->last_event.event_id = -1;
@@ -76,10 +78,13 @@ POname(clCreateCommandQueue)(cl_context context,
   POCL_RETAIN_OBJECT(context);
   POCL_RETAIN_OBJECT(device);
 
+  errcode = CL_SUCCESS;
+  if (device->ops->init_queue)
+    errcode = device->ops->init_queue (command_queue);
+
   if (errcode_ret != NULL)
-    *errcode_ret = CL_SUCCESS;
+    *errcode_ret = errcode;
 
-  pocl_queue_list_insert(command_queue);
   return command_queue;
 
 ERROR:
