@@ -21,8 +21,9 @@
    THE SOFTWARE.
 */
 
-#include "pocl_cl.h"
 #include "devices.h"
+#include "pocl_cl.h"
+#include "pocl_util.h"
 
 /* NOTE: this function is untested! */
 CL_API_ENTRY cl_mem CL_API_CALL
@@ -37,6 +38,8 @@ POname(clCreateSubBuffer)(cl_mem                   buffer,
   int errcode;
   unsigned i;
 
+  HANDLE_IMAGE1D_BUFFER (buffer);
+
   POCL_GOTO_ERROR_COND((buffer == NULL), CL_INVALID_MEM_OBJECT);
 
   POCL_GOTO_ERROR_ON((buffer->parent != NULL), CL_INVALID_MEM_OBJECT,
@@ -65,11 +68,16 @@ POname(clCreateSubBuffer)(cl_mem                   buffer,
 
   POCL_INIT_OBJECT(mem);
   mem->mappings = NULL;
+  mem->destructor_callbacks = NULL;
   mem->parent = buffer;
-
   mem->type = CL_MEM_OBJECT_BUFFER;
   mem->size = info->size;
+  mem->origin = info->origin;
   mem->context = buffer->context;
+  mem->latest_event = NULL;
+  mem->owning_device = buffer->owning_device;
+  mem->is_pipe = CL_FALSE;
+  mem->mem_host_ptr = NULL;
 
   POCL_GOTO_ERROR_ON((buffer->flags & CL_MEM_WRITE_ONLY &&
        flags & (CL_MEM_READ_WRITE | CL_MEM_READ_ONLY)), CL_INVALID_VALUE,
@@ -101,34 +109,15 @@ POname(clCreateSubBuffer)(cl_mem                   buffer,
        "Invalid flags: buffer is CL_MEM_HOST_NO_ACCESS, requested sub-buffer "
        "(CL_MEM_HOST_READ_ONLY | CL_MEM_HOST_WRITE_ONLY)\n");
 
-  if ((flags & CL_MEM_READ_WRITE) |
-      (flags & CL_MEM_READ_ONLY) |
-      (flags & CL_MEM_WRITE_ONLY)) 
-    { 
-      mem->flags =
-        (flags & CL_MEM_READ_WRITE) |
-        (flags & CL_MEM_READ_ONLY) |
-        (flags & CL_MEM_WRITE_ONLY);
-    } else 
-    {
-      mem->flags =
-        (buffer->flags & CL_MEM_READ_WRITE) |
-        (buffer->flags & CL_MEM_READ_ONLY) |
-        (buffer->flags & CL_MEM_WRITE_ONLY);
-    }
+  pocl_cl_mem_inherit_flags (mem, buffer, flags);
 
-  mem->flags = mem->flags |
-    (buffer->flags & CL_MEM_USE_HOST_PTR) |
-    (buffer->flags & CL_MEM_ALLOC_HOST_PTR) |
-    (buffer->flags & CL_MEM_COPY_HOST_PTR);
-
-  if (mem->flags & CL_MEM_USE_HOST_PTR)
+  if (mem->flags & CL_MEM_USE_HOST_PTR || mem->flags & CL_MEM_ALLOC_HOST_PTR)
     {
       mem->mem_host_ptr = (char*)buffer->mem_host_ptr + info->origin;
     }
 
   mem->device_ptrs = (pocl_mem_identifier*)
-    malloc(pocl_num_devices * sizeof(pocl_mem_identifier));
+    malloc(buffer->context->num_devices * sizeof(pocl_mem_identifier));
   if (mem->device_ptrs == NULL)
     {
         errcode = CL_OUT_OF_HOST_MEMORY;
@@ -137,7 +126,7 @@ POname(clCreateSubBuffer)(cl_mem                   buffer,
 
   for (i = 0; i < pocl_num_devices; ++i)
     mem->device_ptrs[i].mem_ptr = NULL;
-  
+
   for (i = 0; i < mem->context->num_devices; ++i)
     {
       device = mem->context->devices[i];
@@ -159,6 +148,8 @@ POname(clCreateSubBuffer)(cl_mem                   buffer,
   POCL_RETAIN_OBJECT(mem->parent);
   POCL_RETAIN_OBJECT(mem->context);
 
+  POCL_MSG_PRINT_INFO ("Created Subbuffer %p, parent %p\n", mem, mem->parent);
+
   if (errcode_ret != NULL)
     *errcode_ret = CL_SUCCESS;
   return mem;
