@@ -24,7 +24,7 @@
 #include "pocl_cl.h"
 #include <assert.h>
 #include "pocl_util.h"
-#include "pocl_img_buf_cpy.h"
+#include "pocl_shared.h"
 
 CL_API_ENTRY cl_int CL_API_CALL
 POname(clEnqueueCopyBufferRect)(cl_command_queue command_queue,
@@ -41,7 +41,7 @@ POname(clEnqueueCopyBufferRect)(cl_command_queue command_queue,
                                 const cl_event *event_wait_list,
                                 cl_event *event) CL_API_SUFFIX__VERSION_1_1
 {
-  return pocl_rect_copy(command_queue,
+  return pocl_rect_copy(command_queue, CL_COMMAND_COPY_BUFFER_RECT,
     src_buffer, CL_FALSE,
     dst_buffer, CL_FALSE,
     src_origin, dst_origin, region,
