@@ -25,7 +25,7 @@
 #include "utlist.h"
 #include <assert.h>
 #include "pocl_util.h"
-#include "clEnqueueMapBuffer.h"
+#include "pocl_shared.h"
 
 CL_API_ENTRY void * CL_API_CALL
 POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
@@ -59,14 +59,14 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
   POCL_GOTO_ERROR_ON((command_queue->context != buffer->context),
     CL_INVALID_CONTEXT, "buffer and command_queue are not from the same context\n");
 
-  POCL_GOTO_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
-    CL_INVALID_EVENT_WAIT_LIST);
-
-  POCL_GOTO_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
-    CL_INVALID_EVENT_WAIT_LIST);
+  errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
+                                        event_wait_list);
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
 
   errcode = pocl_buffer_boundcheck(buffer, offset, size);
-  if (errcode != CL_SUCCESS) goto ERROR;
+  if (errcode != CL_SUCCESS)
+    goto ERROR;
 
   POCL_GOTO_ERROR_ON((buffer->flags & (CL_MEM_HOST_WRITE_ONLY | CL_MEM_HOST_NO_ACCESS) &&
     map_flags & CL_MAP_READ), CL_INVALID_OPERATION, "buffer has been created with "
@@ -90,7 +90,8 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
       goto ERROR;
     }
 
-  if (buffer->flags & CL_MEM_USE_HOST_PTR)
+  if ((buffer->flags & CL_MEM_USE_HOST_PTR)
+      || (buffer->flags & CL_MEM_ALLOC_HOST_PTR))
     {
       /* In this case it should use the given host_ptr + offset as
          the mapping area in the host memory. */   
@@ -103,11 +104,10 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
          the mapping will be stored (the last argument is NULL) in
          the host memory. When the last argument is non-NULL, the
          buffer will be mapped there (assumed it will succeed).  */
-      
-      host_ptr = device->ops->map_mem 
-        (device->data, buffer->device_ptrs[device->dev_id].mem_ptr, offset, 
-         size, 
-         NULL);
+
+      host_ptr = device->ops->map_mem (
+          device->data, buffer->device_ptrs[device->dev_id].mem_ptr, offset,
+          size, NULL);
     }
 
   if (host_ptr == NULL)
@@ -116,10 +116,10 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
       goto ERROR;
     }
 
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MAP_BUFFER,
-                                 event, num_events_in_wait_list,
-                                 event_wait_list);
-
+  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MAP_BUFFER, 
+                                 event, num_events_in_wait_list, 
+                                 event_wait_list, 1, &buffer);
+  
   if (errcode != CL_SUCCESS)
       goto ERROR;
 
@@ -130,8 +130,13 @@ POname(clEnqueueMapBuffer)(cl_command_queue command_queue,
   mapping_info->offset = offset;
   mapping_info->size = size;
   POCL_LOCK_OBJ (buffer);
-  DL_APPEND (buffer->mappings, mapping_info);  
+  DL_APPEND (buffer->mappings, mapping_info);
   POCL_UNLOCK_OBJ (buffer);
+
+  POCL_MSG_PRINT_MEMORY ("Buffer %p New Mapping: host_ptr %p offset %zu\n",
+                         buffer, mapping_info->host_ptr, mapping_info->offset);
+
+  buffer->owning_device = command_queue->device;
   pocl_command_enqueue(command_queue, cmd);
 
   if (blocking_map != CL_TRUE)
@@ -158,21 +163,3 @@ ERROR:
   return NULL;
 }
 POsym(clEnqueueMapBuffer)
-
-void*
-pocl_map_mem_cmd(cl_device_id device, 
-                 cl_mem buffer, 
-                 mem_mapping_t *mapping_info) {
-
-
-  
-  /* The second call ensures the memory is flushed/updated to the
-     host location. */
-  device->ops->map_mem 
-    (device->data, buffer->device_ptrs[device->dev_id].mem_ptr, 
-     mapping_info->offset, mapping_info->size, mapping_info->host_ptr);
-  
-  buffer->map_count++;
-  return mapping_info->host_ptr;
-
-}
