@@ -49,7 +49,6 @@ CL_API_SUFFIX__VERSION_1_0
   cl_device_id device;
   _cl_command_node *cmd = NULL;
   mem_mapping_t *mapping_info = NULL;
-  cl_uint event_i;
 
   POCL_GOTO_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
@@ -66,24 +65,8 @@ CL_API_SUFFIX__VERSION_1_0
   POCL_GOTO_ERROR_ON((!image->is_image), CL_INVALID_MEM_OBJECT,
     "image argument is not an image type cl_mem\n");
 
-  POCL_GOTO_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
-    CL_INVALID_EVENT_WAIT_LIST);
-
-  POCL_GOTO_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
-    CL_INVALID_EVENT_WAIT_LIST);
-
-  for (event_i = 0; event_i < num_events_in_wait_list; ++event_i)
-    {
-      POCL_GOTO_ERROR_COND((event_wait_list[event_i] == NULL), CL_INVALID_EVENT_WAIT_LIST);
-      if (event_i > 0)
-        {
-          POCL_GOTO_ERROR_COND((event_wait_list[event_i]->context 
-                                  != event_wait_list[event_i - 1]->context), 
-                                 CL_INVALID_CONTEXT);
-        }
-    }
-
-  errcode = pocl_check_device_supports_image(image, command_queue);
+  errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
+                                        event_wait_list);
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
@@ -101,16 +84,45 @@ CL_API_SUFFIX__VERSION_1_0
        "image_slice_pitch must be a non-NULL value\n");
 
   /* TODO: more error checks */
-  
-  offset = image->image_channels * image->image_elem_size * origin[0];
-  
-  mapping_info = (mem_mapping_t*) malloc (sizeof (mem_mapping_t));
+  size_t tuned_origin[3]
+      = { origin[0] * image->image_elem_size * image->image_channels,
+          origin[1], origin[2] };
+
+  offset = tuned_origin[0] + tuned_origin[1] * image->image_row_pitch
+           + tuned_origin[2] * image->image_slice_pitch;
+
+  mapping_info = (mem_mapping_t*) calloc (1, sizeof (mem_mapping_t));
   if (mapping_info == NULL)
     {
       errcode = CL_OUT_OF_HOST_MEMORY;
       goto ERROR;
     }
 
+  *image_row_pitch = image->image_row_pitch;
+  if (image_slice_pitch)
+    *image_slice_pitch = image->image_slice_pitch;
+
+  HANDLE_IMAGE1D_BUFFER (image);
+
+  /* CL_INVALID_OPERATION if buffer has been created with
+   * CL_MEM_HOST_WRITE_ONLY
+   * or CL_MEM_HOST_NO_ACCESS and CL_MAP_READ is set in map_flags or
+   *
+   * if buffer has been created with CL_MEM_HOST_READ_ONL or
+   * CL_MEM_HOST_NO_ACCESS
+   * and CL_MAP_WRITE or CL_MAP_WRITE_INVALIDATE_REGION is set in map_flags.
+   */
+
+  POCL_GOTO_ERROR_COND (
+      ((map_flags & CL_MAP_READ)
+       && (image->flags & (CL_MEM_HOST_WRITE_ONLY | CL_MEM_HOST_NO_ACCESS))),
+      CL_INVALID_OPERATION);
+
+  POCL_GOTO_ERROR_COND (
+      ((map_flags & CL_MAP_WRITE)
+       && (image->flags & (CL_MEM_HOST_READ_ONLY | CL_MEM_HOST_NO_ACCESS))),
+      CL_INVALID_OPERATION);
+
   if (image->flags & CL_MEM_USE_HOST_PTR)
     {
       /* In this case it should use the given host_ptr + offset as
@@ -133,7 +145,6 @@ CL_API_SUFFIX__VERSION_1_0
 
   if (map == NULL)
     {
-      POCL_UPDATE_EVENT_COMPLETE(event);
       errcode = CL_MAP_FAILURE;
       goto ERROR;
     }
@@ -147,6 +158,9 @@ CL_API_SUFFIX__VERSION_1_0
   DL_APPEND (image->mappings, mapping_info);
   POCL_UNLOCK_OBJ (image);
 
+  POCL_MSG_PRINT_MEMORY ("Image %p, Mapping: host_ptr %p offset %zu\n", image,
+                         mapping_info->host_ptr, mapping_info->offset);
+
   errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MAP_IMAGE, 
                                  event, num_events_in_wait_list, 
                                  event_wait_list, 1, &image);
@@ -156,8 +170,6 @@ CL_API_SUFFIX__VERSION_1_0
   cmd->command.map.buffer = image;
   cmd->command.map.mapping = mapping_info;
   POname(clRetainMemObject) (image);
-  image->owning_device = command_queue->device;
-
   image->owning_device = command_queue->device;
   pocl_command_enqueue(command_queue, cmd);
 
@@ -166,10 +178,6 @@ CL_API_SUFFIX__VERSION_1_0
       POname(clFinish) (command_queue);
     }
 
-  *image_row_pitch = image->image_row_pitch;
-  if (image_slice_pitch)
-    *image_slice_pitch = image->image_slice_pitch;
-
   if (errcode_ret != NULL)
     (*errcode_ret) = CL_SUCCESS;
 
