@@ -32,8 +32,8 @@ POname(clEnqueueReadImage)(cl_command_queue     command_queue,
                            cl_bool              blocking_read, 
                            const size_t *       origin, /* [3] */
                            const size_t *       region, /* [3] */
-                           size_t               host_row_pitch,
-                           size_t               host_slice_pitch, 
+                           size_t               row_pitch,
+                           size_t               slice_pitch,
                            void *               ptr,
                            cl_uint              num_events_in_wait_list,
                            const cl_event *     event_wait_list,
@@ -52,13 +52,29 @@ CL_API_SUFFIX__VERSION_1_0
   POCL_RETURN_ERROR_ON((command_queue->context != image->context),
     CL_INVALID_CONTEXT, "image and command_queue are not from the same context\n");
 
-  POCL_RETURN_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
-    CL_INVALID_EVENT_WAIT_LIST);
+  POCL_RETURN_ERROR_ON (
+      (!command_queue->device->image_support), CL_INVALID_OPERATION,
+      "Device %s does not support images\n", command_queue->device->long_name);
 
-  POCL_RETURN_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
-    CL_INVALID_EVENT_WAIT_LIST);
+  errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
+                                        event_wait_list);
+  if (errcode != CL_SUCCESS)
+    return errcode;
+
+  POCL_RETURN_ERROR_ON (
+      (image->flags & (CL_MEM_HOST_WRITE_ONLY | CL_MEM_HOST_NO_ACCESS)),
+      CL_INVALID_OPERATION,
+      "image has been created with CL_MEM_HOST_WRITE_ONLY "
+      "or CL_MEM_HOST_NO_ACCESS\n");
+
+  if (image->buffer)
+    POCL_RETURN_ERROR_ON (
+        (image->buffer->flags
+         & (CL_MEM_HOST_WRITE_ONLY | CL_MEM_HOST_NO_ACCESS)),
+        CL_INVALID_OPERATION,
+        "1D Image buffer has been created with CL_MEM_HOST_WRITE_ONLY "
+        "or CL_MEM_HOST_NO_ACCESS\n");
 
-  errcode = pocl_check_device_supports_image(image, command_queue);
   if (errcode != CL_SUCCESS)
     return errcode;
 
@@ -73,23 +89,32 @@ CL_API_SUFFIX__VERSION_1_0
   
   errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_READ_IMAGE,
                                 event, num_events_in_wait_list, 
-                                event_wait_list);
+                                event_wait_list, 1, &image);
   if (errcode != CL_SUCCESS)
     {
       POCL_MEM_FREE(cmd);
       return errcode;
     }
 
-  cmd->command.rw_image.device_ptr = 
-    image->device_ptrs[command_queue->device->dev_id].mem_ptr;
-  cmd->command.rw_image.host_ptr = ptr;
-  memcpy ((cmd->command.rw_image.origin), tuned_origin, 3*sizeof (size_t));
-  memcpy ((cmd->command.rw_image.region), tuned_region, 3*sizeof (size_t));
-  cmd->command.rw_image.rowpitch = image->image_row_pitch;
-  cmd->command.rw_image.slicepitch = image->image_slice_pitch;
-  cmd->command.rw_image.buffer = image;
-  pocl_command_enqueue (command_queue, cmd);
+  cmd->command.read_image.b_rowpitch = image->image_row_pitch;
+  cmd->command.read_image.b_slicepitch = image->image_slice_pitch;
+  cmd->command.read_image.h_rowpitch
+      = (row_pitch ? row_pitch : tuned_region[0]);
+  cmd->command.read_image.h_slicepitch
+      = (slice_pitch ? slice_pitch : (tuned_region[0] * region[1]));
+  memcpy ((cmd->command.read_image.origin), tuned_origin, 3 * sizeof (size_t));
+  memcpy ((cmd->command.read_image.region), tuned_region, 3 * sizeof (size_t));
+
+  HANDLE_IMAGE1D_BUFFER (image);
+
+  cmd->command.read_image.device_ptr
+      = image->device_ptrs[command_queue->device->dev_id].mem_ptr;
+  cmd->command.read_image.host_ptr = ptr;
+  cmd->command.read_image.buffer = image;
+
   POname(clRetainMemObject) (image);  
+  image->owning_device = command_queue->device;
+  pocl_command_enqueue(command_queue, cmd);
 
   if (blocking_read)
     POname(clFinish) (command_queue);
