@@ -49,44 +49,42 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
   POCL_RETURN_ERROR_ON((command_queue->context != buffer->context),
     CL_INVALID_CONTEXT, "buffer and command_queue are not from the same context\n");
 
+  POCL_RETURN_ERROR_ON (
+      (buffer->flags & (CL_MEM_HOST_READ_ONLY | CL_MEM_HOST_NO_ACCESS)),
+      CL_INVALID_OPERATION,
+      "buffer has been created with CL_MEM_HOST_READ_ONLY "
+      "or CL_MEM_HOST_NO_ACCESS\n");
+
   POCL_RETURN_ERROR_COND((ptr == NULL), CL_INVALID_VALUE);
+
   if (pocl_buffer_boundcheck(buffer, offset, cb) != CL_SUCCESS)
     return CL_INVALID_VALUE;
 
-  POCL_RETURN_ERROR_COND((event_wait_list == NULL && num_events_in_wait_list > 0),
-    CL_INVALID_EVENT_WAIT_LIST);
+  errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
+                                        event_wait_list);
+  if (errcode != CL_SUCCESS)
+    return errcode;
 
-  POCL_RETURN_ERROR_COND((event_wait_list != NULL && num_events_in_wait_list == 0),
-    CL_INVALID_EVENT_WAIT_LIST);
-
-  for(i=0; i<num_events_in_wait_list; i++)
-    POCL_RETURN_ERROR_COND((event_wait_list[i] == NULL), CL_INVALID_EVENT_WAIT_LIST);
-
-  device = POCL_REAL_DEV(command_queue->device);
-
-  for (i = 0; i < command_queue->context->num_devices; ++i)
-    {
-        if (command_queue->context->devices[i] == device)
-            break;
-    }
-  assert(i < command_queue->context->num_devices);
+  POCL_CHECK_DEV_IN_CMDQ;
 
   errcode = pocl_create_command (&cmd, command_queue, 
                                  CL_COMMAND_WRITE_BUFFER, 
                                  event, num_events_in_wait_list, 
-                                 event_wait_list);
+                                 event_wait_list, 1, &buffer);
   if (errcode != CL_SUCCESS)
     return errcode;
-  
+
   cmd->command.write.host_ptr = ptr;
   cmd->command.write.device_ptr =
     (char*)buffer->device_ptrs[device->dev_id].mem_ptr;
   cmd->command.write.offset = offset;
   cmd->command.write.cb = cb;
   cmd->command.write.buffer = buffer;
+
   POname(clRetainMemObject) (buffer);
+  buffer->owning_device = command_queue->device;
   pocl_command_enqueue(command_queue, cmd);
-  
+
   if (blocking_write)
     POname(clFinish) (command_queue);
 
