@@ -35,6 +35,8 @@ POname(clGetEventProfilingInfo)(cl_event event,
 
   POCL_RETURN_ERROR_COND((event == NULL), CL_INVALID_EVENT);
 
+  POCL_RETURN_ERROR_ON((event->queue == NULL),
+    CL_PROFILING_INFO_NOT_AVAILABLE, "Cannot return profiling info for user events\n");
   POCL_RETURN_ERROR_ON(((event->queue->properties & CL_QUEUE_PROFILING_ENABLE) == 0),
     CL_PROFILING_INFO_NOT_AVAILABLE, "Cannot return profiling info when profiling "
       "is disabled on the queue\n");
