@@ -42,7 +42,10 @@ POname(clGetMemObjectInfo)(cl_mem      memobj ,
   case CL_MEM_SIZE:
     POCL_RETURN_GETINFO (size_t, memobj->size);
   case CL_MEM_HOST_PTR:
-    POCL_RETURN_GETINFO (void *, memobj->mem_host_ptr);
+    if (memobj->flags & CL_MEM_USE_HOST_PTR)
+      POCL_RETURN_GETINFO (void *, memobj->mem_host_ptr);
+    else
+      POCL_RETURN_GETINFO (void *, NULL);
   case CL_MEM_MAP_COUNT:
     POCL_RETURN_GETINFO (cl_uint, memobj->map_count);
   case CL_MEM_REFERENCE_COUNT:
@@ -54,8 +57,8 @@ POname(clGetMemObjectInfo)(cl_mem      memobj ,
   case CL_MEM_OFFSET:
     if (memobj->parent == NULL)
       POCL_RETURN_GETINFO (size_t, 0);
-
-    POCL_ABORT_UNIMPLEMENTED("clGetMemObjectInfo: CL_MEM_OFFSET in subbuffers");
+    else
+      POCL_RETURN_GETINFO (size_t, memobj->origin);
   }
   return CL_INVALID_VALUE;
 }
