@@ -219,6 +219,10 @@ struct _cl_icd_dispatch pocl_dispatch = {
   NULL, /* &clUnknown176 */
   NULL, /* &clUnknown177 */
 #endif
+#if (OCL_ICD_IDENTIFIED_FUNCTIONS > 129)
+  NULL, /* &clUnknown178 */
+  NULL, /* &clUnknown179 */
+#endif
 };
 
 static struct _cl_platform_id _platforms[1]  = {{&pocl_dispatch}};
