@@ -39,6 +39,8 @@ POname(clGetProgramBuildInfo)(cl_program            program,
 
   POCL_RETURN_ERROR_COND((program == NULL), CL_INVALID_PROGRAM);
 
+  POCL_RETURN_ERROR_COND ((device == NULL), CL_INVALID_DEVICE);
+
   int device_i = pocl_cl_device_to_index(program, device);
   POCL_RETURN_ERROR_ON((device_i < 0), CL_INVALID_DEVICE, "Program does not have "
     "this device in it's device list\n");
@@ -57,31 +59,29 @@ POname(clGetProgramBuildInfo)(cl_program            program,
     
   case CL_PROGRAM_BUILD_LOG:
     {
-      char *build_log;
+      POCL_RETURN_ERROR_ON((program->build_status == CL_BUILD_NONE),
+                           CL_INVALID_PROGRAM,
+                           "Program was not built");
       if (program->main_build_log[0])
-          build_log = strdup(program->main_build_log);
-      else if (program->build_log[device_i])
-          build_log = strdup(program->build_log[device_i]);
-      else
-          build_log = pocl_cache_read_buildlog(program, device_i);
-      if (program->build_status == CL_BUILD_NONE)
-          build_log = "";
-      POCL_RETURN_ERROR_ON((build_log==NULL), CL_OUT_OF_HOST_MEMORY, "failed to read build log");
-
-      size_t const value_size = strlen(build_log) + 1;
-      if (param_value)
-      {
-        if (param_value_size < value_size)
         {
-            POCL_MEM_FREE(build_log);
-            return CL_INVALID_VALUE;
+          POCL_RETURN_GETINFO_STR (program->main_build_log);
         }
-        memcpy(param_value, build_log, value_size);
-      }
-      POCL_MEM_FREE(build_log);
-      if (param_value_size_ret)
-        *param_value_size_ret = value_size;
-      return CL_SUCCESS;
+      else if (program->build_log[device_i])
+        {
+          POCL_RETURN_GETINFO_STR (program->build_log[device_i]);
+        }
+      else
+        {
+          char *build_log = pocl_cache_read_buildlog (program, device_i);
+          if (build_log)
+            POCL_RETURN_GETINFO_STR_FREE (build_log);
+        }
+
+      POCL_RETURN_GETINFO_STR ("");
+    }
+  case CL_PROGRAM_BINARY_TYPE:
+    {
+      POCL_RETURN_GETINFO(cl_program_binary_type, program->binary_type);
     }
   }
   
