@@ -32,18 +32,23 @@ POname(clReleaseKernel)(cl_kernel kernel) CL_API_SUFFIX__VERSION_1_0
   unsigned i;
 
   POCL_RETURN_ERROR_COND((kernel == NULL), CL_INVALID_KERNEL);
-
   POCL_RELEASE_OBJECT (kernel, new_refcount);
+  POCL_MSG_PRINT_REFCOUNTS ("Release kernel %p  %d\n", kernel, new_refcount);
 
   if (new_refcount == 0)
     {
-
-      if (kernel->program != NULL)
+      POCL_MSG_PRINT_REFCOUNTS ("Free kernel %p\n", kernel);
+      cl_program program = kernel->program;
+      /* default kernels are not put into the program->kernels linked list */
+      if ((program != NULL)
+          && (!program->operating_on_default_kernels))
         {
           /* Find the kernel in the program's linked list of kernels */
-          for (pk=&kernel->program->kernels; *pk != NULL; pk = &(*pk)->next)
+          POCL_LOCK_OBJ (program);
+          for (pk = &program->kernels; *pk != NULL; pk = &(*pk)->next)
             {
-              if (*pk == kernel) break;
+              if (*pk == kernel)
+                break;
             }
           if (*pk == NULL)
             {
@@ -51,28 +56,36 @@ POname(clReleaseKernel)(cl_kernel kernel) CL_API_SUFFIX__VERSION_1_0
                  of kernels -- something is wrong */
               return CL_INVALID_VALUE;
             }
-          
+
           /* Remove the kernel from the program's linked list of
              kernels */
           *pk = (*pk)->next;
-          POname(clReleaseProgram) (kernel->program);
-        }
-      
-      POCL_MEM_FREE(kernel->name);
-
-      for (i = 0; i < kernel->num_args; i++)
-        {
-          struct pocl_argument *p = &(kernel->dyn_arguments[i]);
-          if (p->value != NULL)
-            {
-              pocl_aligned_free (p->value);
-              p->value = NULL;
-            }
+          POCL_UNLOCK_OBJ (program);
+          POname (clReleaseProgram) (program);
+          POCL_MSG_PRINT_REFCOUNTS ("Released non-default kernel kernel %p, program %p now has refs: %d \n", kernel, kernel->program, kernel->program->pocl_refcount);
         }
 
-      POCL_MEM_FREE(kernel->dyn_arguments);
-      POCL_MEM_FREE(kernel->reqd_wg_size);
-      POCL_MEM_FREE(kernel);
+      POCL_MEM_FREE (kernel->name);
+      POCL_MEM_FREE (kernel->attributes);
+
+      if (kernel->arg_info)
+        for (i = 0; i < kernel->num_args; i++)
+          {
+            POCL_MEM_FREE (kernel->arg_info[i].name);
+            POCL_MEM_FREE (kernel->arg_info[i].type_name);
+          }
+
+      if (kernel->dyn_arguments)
+        for (i = 0; i < (kernel->num_args + kernel->num_locals); i++)
+          {
+            pocl_aligned_free (kernel->dyn_arguments[i].value);
+          }
+
+      POCL_MEM_FREE (kernel->arg_info);
+      POCL_MEM_FREE (kernel->dyn_arguments);
+      POCL_MEM_FREE (kernel->reqd_wg_size);
+      POCL_DESTROY_OBJECT (kernel);
+      POCL_MEM_FREE (kernel);
     }
   
   return CL_SUCCESS;
