@@ -29,15 +29,18 @@ POname(clReleaseMemObject)(cl_mem memobj) CL_API_SUFFIX__VERSION_1_0
 {
   int new_refcount;
   cl_device_id device_id;
-  cl_context context;
   cl_mem parent = NULL;
   unsigned i;
   mem_mapping_t *mapping, *temp;
+  mem_destructor_callback_t *callback, *next_callback;
+  cl_context context = memobj->context;
 
   POCL_RETURN_ERROR_COND((memobj == NULL), CL_INVALID_MEM_OBJECT);
 
   POCL_RELEASE_OBJECT(memobj, new_refcount);
 
+  POCL_MSG_PRINT_REFCOUNTS ("Release mem obj %p  %d\n", memobj, new_refcount);
+
   /* OpenCL 1.2 Page 118:
 
      After the memobj reference count becomes zero and commands queued for execution on 
@@ -48,14 +51,32 @@ POname(clReleaseMemObject)(cl_mem memobj) CL_API_SUFFIX__VERSION_1_0
 
   if (new_refcount == 0)
     {
+      if (memobj->is_image && (memobj->type == CL_MEM_OBJECT_IMAGE1D_BUFFER))
+        {
+          cl_mem b = memobj->buffer;
+          assert (b);
+          cl_int err = POname (clReleaseMemObject) (b);
+          POCL_MEM_FREE (memobj);
+          return err;
+        }
+      POCL_MSG_PRINT_REFCOUNTS ("Free mem obj %p\n", memobj);
       if (memobj->parent == NULL)
         {
+          cl_device_id shared_mem_owner_dev =
+            memobj->shared_mem_allocation_owner;
+
           for (i = 0; i < memobj->context->num_devices; ++i)
             {
+              /* owner is called last */
+              if (shared_mem_owner_dev == context->devices[i])
+                 continue;
               device_id = memobj->context->devices[i];
               device_id->ops->free(device_id, memobj);
               memobj->device_ptrs[device_id->dev_id].mem_ptr = NULL;
             }
+          if (shared_mem_owner_dev)
+            shared_mem_owner_dev->ops->free (shared_mem_owner_dev, memobj);
+
         }
       DL_FOREACH_SAFE(memobj->mappings, mapping, temp)
         {
@@ -63,10 +84,27 @@ POname(clReleaseMemObject)(cl_mem memobj) CL_API_SUFFIX__VERSION_1_0
         }
       memobj->mappings = NULL;
 
-      context = memobj->context;
       parent = memobj->parent;
 
+      /* Free host mem allocated by the runtime (not for sub buffers) */
+      if (memobj->parent == NULL && memobj->flags & CL_MEM_ALLOC_HOST_PTR
+          && memobj->mem_host_ptr != NULL)
+        {
+          POCL_MEM_FREE(memobj->mem_host_ptr);
+        }
       POCL_MEM_FREE(memobj->device_ptrs);
+
+      /* Fire any registered destructor callbacks */
+      callback = memobj->destructor_callbacks;
+      while (callback)
+      {
+        callback->pfn_notify (memobj, callback->user_data);
+        next_callback = callback->next;
+        free (callback);
+        callback = next_callback;
+      }
+
+      POCL_DESTROY_OBJECT (memobj);
       POCL_MEM_FREE(memobj);
 
       if (parent)
