@@ -33,6 +33,7 @@
 #include "pocl_cl.h"
 #include "pocl_util.h"
 #include "pocl_cache.h"
+#include "pocl_llvm.h"
 #include "devices.h"
 
 CL_API_ENTRY cl_int CL_API_CALL
@@ -45,10 +46,12 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
   POCL_RETURN_ERROR_COND((program == NULL), CL_INVALID_PROGRAM);
 
   POCL_RELEASE_OBJECT (program, new_refcount);
+  POCL_MSG_PRINT_REFCOUNTS ("Release program %p, new refcount: %d, kernel #: %zu \n", program, new_refcount, program->num_kernels);
 
   if (new_refcount == 0)
     {
       cl_context context = program->context;
+      POCL_MSG_PRINT_REFCOUNTS ("Free program %p\n", program);
 
       /* Mark all kernels as having no program.
          FIXME: this should not be needed if the kernels
@@ -59,9 +62,6 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
           k->program = NULL;
         }
 
-      if (program->buildprogram_callback)
-        POCL_MEM_FREE(program->buildprogram_callback);
-
       if (program->devices != program->context->devices)
         POCL_MEM_FREE(program->devices);
 
@@ -73,6 +73,12 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
           POCL_MEM_FREE(program->binaries[i]);
       POCL_MEM_FREE(program->binaries);
 
+      POCL_MEM_FREE(program->pocl_binary_sizes);
+      if (program->pocl_binaries)
+        for (i = 0; i < program->num_devices; ++i)
+          POCL_MEM_FREE(program->pocl_binaries[i]);
+      POCL_MEM_FREE(program->pocl_binaries);
+
       pocl_cache_cleanup_cachedir(program);
 
       if (program->build_log)
@@ -80,9 +86,10 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
           POCL_MEM_FREE(program->build_log[i]);
       POCL_MEM_FREE(program->build_log);
 
+      program->operating_on_default_kernels = 1;
       if (program->num_kernels)
         {
-          for (i=0; i < program->num_kernels; i++)
+          for (i = 0; i < program->num_kernels; i++)
             {
               if (program->kernel_names)
                 POCL_MEM_FREE(program->kernel_names[i]);
@@ -94,7 +101,16 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
         }
 
       POCL_MEM_FREE(program->build_hash);
+      POCL_MEM_FREE(program->compiler_options);
+
+#ifdef OCS_AVAILABLE
+      if (program->llvm_irs)
+        for (i = 0; i < program->num_devices; ++i)
+          pocl_free_llvm_irs (program, i);
+#endif
+
       POCL_MEM_FREE(program->llvm_irs);
+      POCL_DESTROY_OBJECT (program);
       POCL_MEM_FREE(program);
 
       POname(clReleaseContext)(context);
