@@ -29,125 +29,55 @@
 #include "common.h"
 #include "utlist.h"
 #include "devices.h"
+#include "pocl_util.h"
 
+#include <pthread.h>
+#include <utlist.h>
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "pocl_cache.h"
 #include "pocl_timing.h"
+#include "pocl_file_util.h"
+
+#ifdef OCS_AVAILABLE
 #include "pocl_llvm.h"
+#endif
 
-#define max(a,b) (((a) > (b)) ? (a) : (b))
-
-#define COMMAND_LENGTH 2048
-#define WORKGROUP_STRING_LENGTH 1024
+/* default WG size in each dimension & total WG size.
+ * this should be reasonable for CPU */
+#define DEFAULT_WG_SIZE 4096
 
 struct data {
   /* Currently loaded kernel. */
   cl_kernel current_kernel;
   /* Loaded kernel dynamic library handle. */
   lt_dlhandle current_dlhandle;
+  
+  /* List of commands ready to be executed */
+  _cl_command_node * volatile ready_list;
+  /* List of commands not yet ready to be executed */
+  _cl_command_node * volatile command_list;
+  pocl_lock_t cq_lock;      /* Lock for command list related operations */
 };
 
 static const cl_image_format supported_image_formats[] = {
-    { CL_R, CL_SNORM_INT8 },
-    { CL_R, CL_SNORM_INT16 },
-    { CL_R, CL_UNORM_INT8 },
-    { CL_R, CL_UNORM_INT16 },
-    { CL_R, CL_UNORM_SHORT_565 }, 
-    { CL_R, CL_UNORM_SHORT_555 },
-    { CL_R, CL_UNORM_INT_101010 }, 
-    { CL_R, CL_SIGNED_INT8 },
-    { CL_R, CL_SIGNED_INT16 }, 
-    { CL_R, CL_SIGNED_INT32 },
-    { CL_R, CL_UNSIGNED_INT8 }, 
-    { CL_R, CL_UNSIGNED_INT16 },
-    { CL_R, CL_UNSIGNED_INT32 }, 
-    { CL_R, CL_HALF_FLOAT },
-    { CL_R, CL_FLOAT },
-    { CL_Rx, CL_SNORM_INT8 },
-    { CL_Rx, CL_SNORM_INT16 },
-    { CL_Rx, CL_UNORM_INT8 },
-    { CL_Rx, CL_UNORM_INT16 },
-    { CL_Rx, CL_UNORM_SHORT_565 }, 
-    { CL_Rx, CL_UNORM_SHORT_555 },
-    { CL_Rx, CL_UNORM_INT_101010 }, 
-    { CL_Rx, CL_SIGNED_INT8 },
-    { CL_Rx, CL_SIGNED_INT16 }, 
-    { CL_Rx, CL_SIGNED_INT32 },
-    { CL_Rx, CL_UNSIGNED_INT8 }, 
-    { CL_Rx, CL_UNSIGNED_INT16 },
-    { CL_Rx, CL_UNSIGNED_INT32 }, 
-    { CL_Rx, CL_HALF_FLOAT },
-    { CL_Rx, CL_FLOAT },
     { CL_A, CL_SNORM_INT8 },
     { CL_A, CL_SNORM_INT16 },
     { CL_A, CL_UNORM_INT8 },
     { CL_A, CL_UNORM_INT16 },
-    { CL_A, CL_UNORM_SHORT_565 }, 
-    { CL_A, CL_UNORM_SHORT_555 },
-    { CL_A, CL_UNORM_INT_101010 }, 
     { CL_A, CL_SIGNED_INT8 },
     { CL_A, CL_SIGNED_INT16 }, 
     { CL_A, CL_SIGNED_INT32 },
     { CL_A, CL_UNSIGNED_INT8 }, 
     { CL_A, CL_UNSIGNED_INT16 },
     { CL_A, CL_UNSIGNED_INT32 }, 
-    { CL_A, CL_HALF_FLOAT },
     { CL_A, CL_FLOAT },
-    { CL_RG, CL_SNORM_INT8 },
-    { CL_RG, CL_SNORM_INT16 },
-    { CL_RG, CL_UNORM_INT8 },
-    { CL_RG, CL_UNORM_INT16 },
-    { CL_RG, CL_UNORM_SHORT_565 }, 
-    { CL_RG, CL_UNORM_SHORT_555 },
-    { CL_RG, CL_UNORM_INT_101010 }, 
-    { CL_RG, CL_SIGNED_INT8 },
-    { CL_RG, CL_SIGNED_INT16 }, 
-    { CL_RG, CL_SIGNED_INT32 },
-    { CL_RG, CL_UNSIGNED_INT8 }, 
-    { CL_RG, CL_UNSIGNED_INT16 },
-    { CL_RG, CL_UNSIGNED_INT32 }, 
-    { CL_RG, CL_HALF_FLOAT },
-    { CL_RG, CL_FLOAT },
-    { CL_RGx, CL_SNORM_INT8 },
-    { CL_RGx, CL_SNORM_INT16 },
-    { CL_RGx, CL_UNORM_INT8 },
-    { CL_RGx, CL_UNORM_INT16 },
-    { CL_RGx, CL_UNORM_SHORT_565 }, 
-    { CL_RGx, CL_UNORM_SHORT_555 },
-    { CL_RGx, CL_UNORM_INT_101010 }, 
-    { CL_RGx, CL_SIGNED_INT8 },
-    { CL_RGx, CL_SIGNED_INT16 }, 
-    { CL_RGx, CL_SIGNED_INT32 },
-    { CL_RGx, CL_UNSIGNED_INT8 }, 
-    { CL_RGx, CL_UNSIGNED_INT16 },
-    { CL_RGx, CL_UNSIGNED_INT32 }, 
-    { CL_RGx, CL_HALF_FLOAT },
-    { CL_RGx, CL_FLOAT },
-    { CL_RA, CL_SNORM_INT8 },
-    { CL_RA, CL_SNORM_INT16 },
-    { CL_RA, CL_UNORM_INT8 },
-    { CL_RA, CL_UNORM_INT16 },
-    { CL_RA, CL_UNORM_SHORT_565 }, 
-    { CL_RA, CL_UNORM_SHORT_555 },
-    { CL_RA, CL_UNORM_INT_101010 }, 
-    { CL_RA, CL_SIGNED_INT8 },
-    { CL_RA, CL_SIGNED_INT16 }, 
-    { CL_RA, CL_SIGNED_INT32 },
-    { CL_RA, CL_UNSIGNED_INT8 }, 
-    { CL_RA, CL_UNSIGNED_INT16 },
-    { CL_RA, CL_UNSIGNED_INT32 }, 
-    { CL_RA, CL_HALF_FLOAT },
-    { CL_RA, CL_FLOAT },
     { CL_RGBA, CL_SNORM_INT8 },
     { CL_RGBA, CL_SNORM_INT16 },
     { CL_RGBA, CL_UNORM_INT8 },
     { CL_RGBA, CL_UNORM_INT16 },
-    { CL_RGBA, CL_UNORM_SHORT_565 }, 
-    { CL_RGBA, CL_UNORM_SHORT_555 },
-    { CL_RGBA, CL_UNORM_INT_101010 }, 
     { CL_RGBA, CL_SIGNED_INT8 },
     { CL_RGBA, CL_SIGNED_INT16 }, 
     { CL_RGBA, CL_SIGNED_INT32 },
@@ -156,24 +86,6 @@ static const cl_image_format supported_image_formats[] = {
     { CL_RGBA, CL_UNSIGNED_INT32 }, 
     { CL_RGBA, CL_HALF_FLOAT },
     { CL_RGBA, CL_FLOAT },
-    { CL_INTENSITY, CL_UNORM_INT8 }, 
-    { CL_INTENSITY, CL_UNORM_INT16 }, 
-    { CL_INTENSITY, CL_SNORM_INT8 }, 
-    { CL_INTENSITY, CL_SNORM_INT16 }, 
-    { CL_INTENSITY, CL_HALF_FLOAT }, 
-    { CL_INTENSITY, CL_FLOAT },
-    { CL_LUMINANCE, CL_UNORM_INT8 }, 
-    { CL_LUMINANCE, CL_UNORM_INT16 }, 
-    { CL_LUMINANCE, CL_SNORM_INT8 }, 
-    { CL_LUMINANCE, CL_SNORM_INT16 }, 
-    { CL_LUMINANCE, CL_HALF_FLOAT }, 
-    { CL_LUMINANCE, CL_FLOAT },
-    { CL_RGB, CL_UNORM_SHORT_565 }, 
-    { CL_RGB, CL_UNORM_SHORT_555 },
-    { CL_RGB, CL_UNORM_INT_101010 }, 
-    { CL_RGBx, CL_UNORM_SHORT_565 }, 
-    { CL_RGBx, CL_UNORM_SHORT_555 },
-    { CL_RGBx, CL_UNORM_INT_101010 }, 
     { CL_ARGB, CL_SNORM_INT8 },
     { CL_ARGB, CL_UNORM_INT8 },
     { CL_ARGB, CL_SIGNED_INT8 },
@@ -193,6 +105,7 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->init_device_infos = pocl_basic_init_device_infos;
   ops->probe = pocl_basic_probe;
   ops->uninit = pocl_basic_uninit;
+  ops->reinit = pocl_basic_reinit;
   ops->init = pocl_basic_init;
   ops->alloc_mem_obj = pocl_basic_alloc_mem_obj;
   ops->free = pocl_basic_free;
@@ -206,16 +119,38 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->fill_rect = pocl_basic_fill_rect;
   ops->memfill = pocl_basic_memfill;
   ops->map_mem = pocl_basic_map_mem;
+  ops->compile_kernel = pocl_basic_compile_kernel;
   ops->unmap_mem = pocl_basic_unmap_mem;
-  ops->compile_submitted_kernels = pocl_basic_compile_submitted_kernels;
   ops->run = pocl_basic_run;
   ops->run_native = pocl_basic_run_native;
   ops->get_timer_value = pocl_basic_get_timer_value;
   ops->get_supported_image_formats = pocl_basic_get_supported_image_formats;
+  ops->join = pocl_basic_join;
+  ops->submit = pocl_basic_submit;
+  ops->broadcast = pocl_basic_broadcast;
+  ops->notify = pocl_basic_notify;
+  ops->flush = pocl_basic_flush;
+  ops->build_hash = pocl_basic_build_hash;
 }
 
+char *
+pocl_basic_build_hash (cl_device_id device)
+{
+  char* res = calloc(1000, sizeof(char));
+#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
+  char *name = get_cpu_name ();
+  snprintf (res, 1000, "basic-%s-%s", HOST_DEVICE_BUILD_HASH, name);
+  POCL_MEM_FREE (name);
+#else
+  snprintf (res, 1000, "basic-%s", HOST_DEVICE_BUILD_HASH);
+#endif
+  return res;
+}
+
+static cl_device_partition_property basic_partition_properties[1] = { 0 };
+
 void
-pocl_basic_init_device_infos(struct _cl_device_id* dev)
+pocl_basic_init_device_infos(unsigned j, struct _cl_device_id* dev)
 {
   dev->type = CL_DEVICE_TYPE_CPU;
   dev->vendor_id = 0;
@@ -230,10 +165,20 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
     the SIMD lanes times the vector units, but not more than
     that to avoid stack overflow and cache trashing.
   */
-  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1] =
-	  dev->max_work_item_sizes[2] = dev->max_work_group_size = 1024*4;
+  int max_wg
+      = pocl_get_int_option ("POCL_MAX_WORK_GROUP_SIZE", DEFAULT_WG_SIZE);
+  assert (max_wg > 0);
+  max_wg = min (max_wg, DEFAULT_WG_SIZE);
+  if (max_wg < 0)
+    max_wg = DEFAULT_WG_SIZE;
+
+  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1]
+      = dev->max_work_item_sizes[2] = dev->max_work_group_size = max_wg;
 
   dev->preferred_wg_size_multiple = 8;
+#ifdef OCS_AVAILABLE
+  cpu_setup_vector_widths (dev);
+#else
   dev->preferred_vector_width_char = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_CHAR;
   dev->preferred_vector_width_short = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_SHORT;
   dev->preferred_vector_width_int = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_INT;
@@ -242,20 +187,21 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->preferred_vector_width_double = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_DOUBLE;
   dev->preferred_vector_width_half = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_HALF;
   /* TODO: figure out what the difference between preferred and native widths are */
-  dev->native_vector_width_char = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_CHAR;
-  dev->native_vector_width_short = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_SHORT;
-  dev->native_vector_width_int = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_INT;
-  dev->native_vector_width_long = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_LONG;
-  dev->native_vector_width_float = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_FLOAT;
-  dev->native_vector_width_double = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_DOUBLE;
-  dev->native_vector_width_half = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_HALF;
+  dev->native_vector_width_char = POCL_DEVICES_NATIVE_VECTOR_WIDTH_CHAR;
+  dev->native_vector_width_short = POCL_DEVICES_NATIVE_VECTOR_WIDTH_SHORT;
+  dev->native_vector_width_int = POCL_DEVICES_NATIVE_VECTOR_WIDTH_INT;
+  dev->native_vector_width_long = POCL_DEVICES_NATIVE_VECTOR_WIDTH_LONG;
+  dev->native_vector_width_float = POCL_DEVICES_NATIVE_VECTOR_WIDTH_FLOAT;
+  dev->native_vector_width_double = POCL_DEVICES_NATIVE_VECTOR_WIDTH_DOUBLE;
+  dev->native_vector_width_half = POCL_DEVICES_NATIVE_VECTOR_WIDTH_HALF;
+#endif
+
   dev->max_clock_frequency = 0;
   dev->address_bits = POCL_DEVICE_ADDRESS_BITS;
-
   dev->image_support = CL_TRUE;
-  /* Use the minimum values until we get a more sensible
-     upper limit from somewhere. */
-  dev->max_read_image_args = dev->max_write_image_args = 128;
+  /* Use the minimum values until we get a more sensible upper limit from
+     somewhere. */
+  dev->max_read_image_args = dev->max_write_image_args = dev->max_read_write_image_args = 128;
   dev->image2d_max_width = dev->image2d_max_height = 8192;
   dev->image3d_max_width = dev->image3d_max_height = dev->image3d_max_depth = 2048;
   dev->image_max_buffer_size = 65536;
@@ -264,18 +210,40 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->max_constant_args = 8;
 
   dev->max_mem_alloc_size = 0;
-
   dev->max_parameter_size = 1024;
   dev->min_data_type_align_size = MAX_EXTENDED_ALIGNMENT; // this is in bytes
   dev->mem_base_addr_align = MAX_EXTENDED_ALIGNMENT*8; // this is in bits
   dev->half_fp_config = 0;
   dev->single_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
-  dev->double_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
+#ifdef __x86_64__
+  dev->single_fp_config |= (CL_FP_DENORM | CL_FP_ROUND_TO_INF
+                            | CL_FP_ROUND_TO_ZERO
+                            | CL_FP_CORRECTLY_ROUNDED_DIVIDE_SQRT);
+#ifdef OCS_AVAILABLE
+  if (cpu_has_fma())
+    dev->single_fp_config |= CL_FP_FMA;
+#endif
+#endif
+
+#ifdef _CL_DISABLE_DOUBLE
+  dev->double_fp_config = 0;
+#else
+  /* TODO: all of these are the minimum mandated, but not all CPUs may actually
+   * support all of them. */
+  dev->double_fp_config = CL_FP_FMA | CL_FP_ROUND_TO_NEAREST
+                          | CL_FP_ROUND_TO_ZERO | CL_FP_ROUND_TO_INF
+                          | CL_FP_INF_NAN | CL_FP_DENORM;
+  /* this is a workaround for issue 28 in https://github.com/Oblomov/clinfo
+   * https://github.com/Oblomov/clinfo/issues/28 */
+  dev->double_fp_config |= CL_FP_CORRECTLY_ROUNDED_DIVIDE_SQRT;
+#endif
+
   dev->global_mem_cache_type = CL_NONE;
   dev->global_mem_cacheline_size = 0;
   dev->global_mem_cache_size = 0;
   dev->global_mem_size = 0;
   dev->max_constant_buffer_size = 0;
+  dev->max_constant_args = 8;
   dev->local_mem_type = CL_GLOBAL;
   dev->local_mem_size = 0;
   dev->error_correction_support = CL_FALSE;
@@ -287,15 +255,23 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->available = CL_TRUE;
   dev->compiler_available = CL_TRUE;
   dev->spmd = CL_FALSE;
+  dev->workgroup_pass = CL_TRUE;
   dev->execution_capabilities = CL_EXEC_KERNEL | CL_EXEC_NATIVE_KERNEL;
   dev->platform = 0;
 
   dev->parent_device = NULL;
-  // basic does not support partitioning
+  /* These two are only used for subdevices.
+   * Each subdevice has these two setup when created.
+   * The subdevice will then use these CUs:
+   *  [start, start+1, ..., start+count-1]
+   * this may not work with more complicated partitioning schemes,
+   * but is good enough for now. */
+  dev->core_start = 0;
+  dev->core_count = 0;
+  /* basic does not support partitioning */
   dev->max_sub_devices = 1;
   dev->num_partition_properties = 1;
-  dev->partition_properties = calloc(dev->num_partition_properties,
-    sizeof(cl_device_partition_property));
+  dev->partition_properties = basic_partition_properties;
   dev->num_partition_types = 0;
   dev->partition_type = NULL;
 
@@ -311,6 +287,8 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
      ensure that there is no more than a single space between
      identifiers. */
 
+  dev->global_as_id = dev->local_as_id = dev->constant_as_id = 0;
+
   dev->should_allocate_svm = 0;
   /* OpenCL 2.0 properties */
   dev->svm_caps = CL_DEVICE_SVM_COARSE_GRAIN_BUFFER
@@ -327,18 +305,27 @@ pocl_basic_init_device_infos(struct _cl_device_id* dev)
   dev->on_dev_queue_props = CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE
                                | CL_QUEUE_PROFILING_ENABLE;
   dev->on_host_queue_props = CL_QUEUE_PROFILING_ENABLE;
-
-
   dev->extensions = HOST_DEVICE_EXTENSIONS;
-
-  dev->llvm_target_triplet = OCL_KERNEL_TARGET;
-#ifdef POCL_BUILT_WITH_CMAKE
-  dev->llvm_cpu = get_cpu_name();
-#else
-  dev->llvm_cpu = OCL_KERNEL_TARGET_CPU;
-#endif
   dev->has_64bit_long = 1;
   dev->autolocals_to_args = 1;
+
+#ifdef OCS_AVAILABLE
+
+  dev->llvm_target_triplet = OCL_KERNEL_TARGET;
+#ifdef HOST_CPU_FORCED
+  dev->llvm_cpu = OCL_KERNEL_TARGET_CPU;
+#else
+  dev->llvm_cpu = get_cpu_name();
+#endif
+
+  if(dev->llvm_cpu && (!strcmp(dev->llvm_cpu, "(unknown)")))
+    dev->llvm_cpu = OCL_KERNEL_TARGET_CPU;
+
+#else /* No compiler, no CPU info */
+  dev->llvm_cpu = NULL;
+  dev->llvm_target_triplet = "";
+#endif
+
 }
 
 unsigned int
@@ -355,23 +342,25 @@ pocl_basic_probe(struct pocl_device_ops *ops)
 
 
 
-void
-pocl_basic_init (cl_device_id device, const char* parameters)
+cl_int
+pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
 {
   struct data *d;
-  static int global_mem_id;
+  cl_int ret = CL_SUCCESS;
+  int err;
   static int first_basic_init = 1;
-  static int device_number = 0;
-  
+
   if (first_basic_init)
     {
+      pocl_init_dlhandle_cache();
       first_basic_init = 0;
-      global_mem_id = device->dev_id;
     }
-  device->global_mem_id = global_mem_id;
+  device->global_mem_id = 0;
 
   d = (struct data *) calloc (1, sizeof (struct data));
-  
+  if (d == NULL)
+    return CL_OUT_OF_HOST_MEMORY;
+
   d->current_kernel = NULL;
   d->current_dlhandle = 0;
   device->data = d;
@@ -381,7 +370,11 @@ pocl_basic_init (cl_device_id device, const char* parameters)
      initialize global_mem_size which it is not yet. Just put 
      a nonzero there for now. */
   device->global_mem_size = 1;
-  pocl_topology_detect_device_info(device);
+  err = pocl_topology_detect_device_info(device);
+  if (err)
+    ret = CL_INVALID_DEVICE;
+
+  POCL_INIT_LOCK (d->cq_lock);
   pocl_cpuinfo_detect_device_info(device);
   pocl_set_buffer_image_limits(device);
 
@@ -392,8 +385,7 @@ pocl_basic_init (cl_device_id device, const char* parameters)
     device->vendor_id =
       magic[0] | magic[1] << 8 | magic[2] << 16 | magic[3] << 24;
 
-  device->vendor_id += device_number;
-  device_number++;
+  device->vendor_id += j;
 
   /* The basic driver represents only one "compute unit" as
      it doesn't exploit multiple hardware threads. Multiple
@@ -401,66 +393,86 @@ pocl_basic_init (cl_device_id device, const char* parameters)
      using multiple OpenCL devices. */
   device->max_compute_units = 1;
 
-  if(!strcmp(device->llvm_cpu, "(unknown)"))
-    device->llvm_cpu = NULL;
-
-  // work-around LLVM bug where sizeof(long)=4
-  #ifdef _CL_DISABLE_LONG
-  device->has_64bit_long=0;
-  #endif
+  return ret;
 }
 
 cl_int
-pocl_basic_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
+pocl_basic_alloc_mem_obj (cl_device_id device, cl_mem mem_obj, void* host_ptr)
 {
   void *b = NULL;
   cl_mem_flags flags = mem_obj->flags;
-
-  /* if memory for this global memory is not yet allocated -> do it */
-  if (mem_obj->device_ptrs[device->global_mem_id].mem_ptr == NULL)
+  unsigned i;
+  POCL_MSG_PRINT_MEMORY (" mem %p, dev %d\n", mem_obj, device->dev_id);
+  /* check if some driver has already allocated memory for this mem_obj 
+     in our global address space, and use that*/
+  for (i = 0; i < mem_obj->context->num_devices; ++i)
     {
-      if (flags & CL_MEM_USE_HOST_PTR)
-        {
-          // mem_host_ptr must be non-NULL
-          assert(mem_obj->mem_host_ptr != NULL);
-          b = mem_obj->mem_host_ptr;
-        }
-      else
-        {
-          b = pocl_memalign_alloc_global_mem(device, MAX_EXTENDED_ALIGNMENT, mem_obj->size);
-          if (b == NULL)
-            return CL_MEM_OBJECT_ALLOCATION_FAILURE;
-        }
+      if (!mem_obj->device_ptrs[i].available)
+        continue;
 
-      if (flags & CL_MEM_COPY_HOST_PTR)
+      if (mem_obj->device_ptrs[i].global_mem_id == device->global_mem_id
+          && mem_obj->device_ptrs[i].mem_ptr != NULL)
         {
-          // mem_host_ptr must be non-NULL
-          assert(mem_obj->mem_host_ptr != NULL);
-          memcpy (b, mem_obj->mem_host_ptr, mem_obj->size);
+          mem_obj->device_ptrs[device->dev_id].mem_ptr =
+            mem_obj->device_ptrs[i].mem_ptr;
+          POCL_MSG_PRINT_MEMORY (
+              "mem %p dev %d, using already allocated mem\n", mem_obj,
+              device->dev_id);
+          return CL_SUCCESS;
         }
-
-      mem_obj->device_ptrs[device->global_mem_id].mem_ptr = b;
-      mem_obj->device_ptrs[device->global_mem_id].global_mem_id = 
-        device->global_mem_id;
     }
-  /* copy already allocated global mem info to devices own slot */
-  mem_obj->device_ptrs[device->dev_id] = 
-    mem_obj->device_ptrs[device->global_mem_id];
+
+  /* memory for this global memory is not yet allocated -> do it */
+  if (flags & CL_MEM_USE_HOST_PTR)
+    {
+      // mem_host_ptr must be non-NULL
+      assert(host_ptr != NULL);
+      b = host_ptr;
+    }
+  else
+    {
+      POCL_MSG_PRINT_MEMORY ("!USE_HOST_PTR\n");
+      b = pocl_memalign_alloc_global_mem (device, MAX_EXTENDED_ALIGNMENT,
+                                          mem_obj->size);
+      if (b==NULL)
+        return CL_MEM_OBJECT_ALLOCATION_FAILURE;
+
+      mem_obj->shared_mem_allocation_owner = device;
+    }
+
+  /* use this dev mem allocation as host ptr */
+  if ((flags & CL_MEM_ALLOC_HOST_PTR) && (mem_obj->mem_host_ptr == NULL))
+    mem_obj->mem_host_ptr = b;
+
+  if (flags & CL_MEM_COPY_HOST_PTR)
+    {
+      POCL_MSG_PRINT_MEMORY ("COPY_HOST_PTR\n");
+      // mem_host_ptr must be non-NULL
+      assert(host_ptr != NULL);
+      memcpy (b, host_ptr, mem_obj->size);
+    }
+
+  mem_obj->device_ptrs[device->dev_id].mem_ptr = b;
 
   return CL_SUCCESS;
 }
 
+
 void
 pocl_basic_free (cl_device_id device, cl_mem memobj)
 {
   cl_mem_flags flags = memobj->flags;
 
-  if (flags & CL_MEM_USE_HOST_PTR)
+  /* aloocation owner executes freeing */
+  if (flags & CL_MEM_USE_HOST_PTR ||
+      memobj->shared_mem_allocation_owner != device)
     return;
 
   void* ptr = memobj->device_ptrs[device->dev_id].mem_ptr;
   size_t size = memobj->size;
 
+  if (memobj->flags | CL_MEM_ALLOC_HOST_PTR)
+    memobj->mem_host_ptr = NULL;
   pocl_free_global_mem(device, ptr, size);
 }
 
@@ -572,6 +584,15 @@ pocl_basic_run
       *(void **)(arguments[i]) = pocl_memalign_alloc(MAX_EXTENDED_ALIGNMENT, al->size);
     }
 
+  pc->local_size[0] = cmd->command.run.local_x;
+  pc->local_size[1] = cmd->command.run.local_y;
+  pc->local_size[2] = cmd->command.run.local_z;
+
+  unsigned rm = pocl_save_rm ();
+  pocl_set_default_rm ();
+  unsigned ftz = pocl_save_ftz ();
+  pocl_set_ftz (kernel->program->flush_denorms);
+
   for (z = 0; z < pc->num_groups[2]; ++z)
     {
       for (y = 0; y < pc->num_groups[1]; ++y)
@@ -587,6 +608,10 @@ pocl_basic_run
             }
         }
     }
+
+  pocl_restore_rm (rm);
+  pocl_restore_ftz (ftz);
+
   for (i = 0; i < kernel->num_args; ++i)
     {
       if (kernel->arg_info[i].is_local)
@@ -613,6 +638,8 @@ pocl_basic_run
       POCL_MEM_FREE(arguments[i]);
     }
   free(arguments);
+
+  pocl_release_dlhandle_cache (cmd);
 }
 
 void
@@ -759,6 +786,10 @@ void pocl_basic_memfill(void *ptr,
   size_t i;
   unsigned j;
 
+  /* memfill size is in bytes, we wanto make it into elements */
+  size /= pattern_size;
+  offset /= pattern_size;
+
   switch (pattern_size)
     {
     case 1:
@@ -791,7 +822,7 @@ void pocl_basic_memfill(void *ptr,
       break;
     case 16:
       {
-      uint64_t * p = (uint64_t*)ptr + offset;
+      uint64_t * p = (uint64_t*)ptr + (offset << 1);
       for (i = 0; i < size; i++)
         for (j = 0; j < 2; j++)
           p[(i<<1) + j] = *((uint64_t*)pattern + j);
@@ -799,7 +830,7 @@ void pocl_basic_memfill(void *ptr,
       break;
     case 32:
       {
-      uint64_t * p = (uint64_t*)ptr + offset;
+      uint64_t * p = (uint64_t*)ptr + (offset << 2);
       for (i = 0; i < size; i++)
         for (j = 0; j < 4; j++)
           p[(i<<2) + j] = *((uint64_t*)pattern + j);
@@ -807,7 +838,7 @@ void pocl_basic_memfill(void *ptr,
       break;
     case 64:
       {
-      uint64_t * p = (uint64_t*)ptr + offset;
+      uint64_t * p = (uint64_t*)ptr + (offset << 3);
       for (i = 0; i < size; i++)
         for (j = 0; j < 8; j++)
           p[(i<<3) + j] = *((uint64_t*)pattern + j);
@@ -815,7 +846,7 @@ void pocl_basic_memfill(void *ptr,
       break;
     case 128:
       {
-      uint64_t * p = (uint64_t*)ptr + offset;
+      uint64_t * p = (uint64_t*)ptr + (offset << 4);
       for (i = 0; i < size; i++)
         for (j = 0; j < 16; j++)
           p[(i<<4) + j] = *((uint64_t*)pattern + j);
@@ -832,26 +863,60 @@ pocl_basic_map_mem (void *data, void *buf_ptr,
                       size_t offset, size_t size,
                       void *host_ptr) 
 {
-  /* All global pointers of the pthread/CPU device are in 
-     the host address space already, and up to date. */
-  if (host_ptr != NULL) return host_ptr;
+  /* If the buffer was allocated with CL_MEM_ALLOC_HOST_PTR |
+   * CL_MEM_COPY_HOST_PTR,
+   * the host_ptr is not the same memory as pocl's device_ptrs[], and we need
+   * to copy pocl's buffer content back to host_ptr. */
+  if ((host_ptr != NULL) && (host_ptr != (buf_ptr + offset)))
+    {
+      POCL_MSG_PRINT_MEMORY ("device: MAP memcpy() "
+                             "buf_ptr %p + offset %zu to host_ptr %p\n",
+                             buf_ptr, offset, host_ptr);
+      memcpy ((char *)host_ptr, (char *)buf_ptr + offset, size);
+    }
   return (char*)buf_ptr + offset;
 }
 
 void* pocl_basic_unmap_mem(void *data, void *host_ptr,
                            void *device_start_ptr,
-                           size_t size)
+                           size_t offset, size_t size)
 {
-  return host_ptr;
+  /* If the buffer was allocated with CL_MEM_ALLOC_HOST_PTR |
+   * CL_MEM_COPY_HOST_PTR,
+   * the host_ptr is not the same memory as pocl's device_ptrs[], and we need
+   * to copy host_ptr content back to  pocl's device_ptrs[]. */
+  if ((host_ptr != NULL) && (host_ptr != (device_start_ptr + offset)))
+    {
+      POCL_MSG_PRINT_MEMORY ("device: UNMAP memcpy() "
+                             "host_ptr %p to buf_ptr %p + offset %zu\n",
+                             host_ptr, device_start_ptr, offset);
+      memcpy ((char *)device_start_ptr + offset, (char *)host_ptr, size);
+    }
+  return (char *)host_ptr;
 }
 
-
-void
+cl_int
 pocl_basic_uninit (cl_device_id device)
 {
   struct data *d = (struct data*)device->data;
+  POCL_DESTROY_LOCK (d->cq_lock);
   POCL_MEM_FREE(d);
   device->data = NULL;
+  return CL_SUCCESS;
+}
+
+cl_int
+pocl_basic_reinit (cl_device_id device)
+{
+  struct data *d = (struct data *)calloc (1, sizeof (struct data));
+  if (d == NULL)
+    return CL_OUT_OF_HOST_MEMORY;
+
+  d->current_kernel = NULL;
+  d->current_dlhandle = 0;
+  POCL_INIT_LOCK (d->cq_lock);
+  device->data = d;
+  return CL_SUCCESS;
 }
 
 cl_ulong
@@ -874,70 +939,103 @@ pocl_basic_get_supported_image_formats (cl_mem_flags flags,
     return CL_SUCCESS; 
 }
 
-typedef struct compiler_cache_item compiler_cache_item;
-struct compiler_cache_item
+static void basic_command_scheduler (struct data *d) 
 {
-  char *tmp_dir;
-  char *function_name;
-  pocl_workgroup wg;
-  compiler_cache_item *next;
-};
-
-static compiler_cache_item *compiler_cache;
-static pocl_lock_t compiler_cache_lock;
-
-void check_compiler_cache (_cl_command_node *cmd)
-{
-  char workgroup_string[WORKGROUP_STRING_LENGTH];
-  lt_dlhandle dlhandle;
-  compiler_cache_item *ci = NULL;
+  _cl_command_node *node;
   
-  if (compiler_cache == NULL)
-    POCL_INIT_LOCK (compiler_cache_lock);
-
-  POCL_LOCK (compiler_cache_lock);
-  LL_FOREACH (compiler_cache, ci)
+  /* execute commands from ready list */
+  while ((node = d->ready_list))
     {
-      if (strcmp (ci->tmp_dir, cmd->command.run.tmp_dir) == 0 &&
-          strcmp (ci->function_name, 
-                  cmd->command.run.kernel->name) == 0)
-        {
-          POCL_UNLOCK (compiler_cache_lock);
-          cmd->command.run.wg = ci->wg;
-          return;
-        }
+      assert (pocl_command_is_ready(node->event));
+      assert (node->event->status == CL_SUBMITTED);
+      CDL_DELETE (d->ready_list, node);
+      POCL_UNLOCK (d->cq_lock);
+      pocl_exec_command (node);
+      POCL_LOCK (d->cq_lock);
     }
 
-  ci = (compiler_cache_item*) malloc (sizeof (compiler_cache_item));
-  ci->next = NULL;
-  ci->tmp_dir = strdup(cmd->command.run.tmp_dir);
-  ci->function_name = strdup (cmd->command.run.kernel->name);
-  const char* module_fn = llvm_codegen (cmd->command.run.tmp_dir,
-                                        cmd->command.run.kernel,
-                                        cmd->device);
-  dlhandle = lt_dlopen (module_fn);
-  if (dlhandle == NULL)
-    {
-      printf ("pocl error: lt_dlopen(\"%s\") failed with '%s'.\n", 
-              module_fn, lt_dlerror());
-      printf ("note: missing symbols in the kernel binary might be" 
-              "reported as 'file not found' errors.\n");
-      abort();
-    }
-  snprintf (workgroup_string, WORKGROUP_STRING_LENGTH,
-            "_pocl_launcher_%s_workgroup", cmd->command.run.kernel->name);
-  cmd->command.run.wg = ci->wg = 
-    (pocl_workgroup) lt_dlsym (dlhandle, workgroup_string);
-
-  LL_APPEND (compiler_cache, ci);
-  POCL_UNLOCK (compiler_cache_lock);
-
+  return;
 }
 
 void
-pocl_basic_compile_submitted_kernels (_cl_command_node *cmd)
+pocl_basic_submit (_cl_command_node *node, cl_command_queue cq)
 {
-  if (cmd->type == CL_COMMAND_NDRANGE_KERNEL)
-    check_compiler_cache (cmd);
+  struct data *d = node->device->data;
 
+  if (node != NULL && node->type == CL_COMMAND_NDRANGE_KERNEL)
+    pocl_check_dlhandle_cache (node, 1);
+
+  node->ready = 1;
+  POCL_LOCK (d->cq_lock);
+  pocl_command_push(node, &d->ready_list, &d->command_list);
+
+  POCL_UNLOCK_OBJ (node->event);
+  basic_command_scheduler (d);
+  POCL_UNLOCK (d->cq_lock);
+
+  return;
+}
+
+void pocl_basic_flush (cl_device_id device, cl_command_queue cq)
+{
+  struct data *d = (struct data*)device->data;
+
+  POCL_LOCK (d->cq_lock);
+  basic_command_scheduler (d);
+  POCL_UNLOCK (d->cq_lock);
+}
+
+void
+pocl_basic_join(cl_device_id device, cl_command_queue cq)
+{
+  struct data *d = (struct data*)device->data;
+
+  POCL_LOCK (d->cq_lock);
+  basic_command_scheduler (d);
+  POCL_UNLOCK (d->cq_lock);
+
+  return;
+}
+
+void
+pocl_basic_notify (cl_device_id device, cl_event event, cl_event finished)
+{
+  struct data *d = (struct data*)device->data;
+  _cl_command_node * volatile node = event->command;
+
+  if (finished->status < CL_COMPLETE)
+    {
+      POCL_UPDATE_EVENT_FAILED (event);
+      return;
+    }
+
+  if (!node->ready)
+    return;
+
+  if (pocl_command_is_ready (event))
+    {
+      if (event->status == CL_QUEUED)
+        {
+          POCL_UPDATE_EVENT_SUBMITTED (event);
+          POCL_LOCK (d->cq_lock);
+          CDL_DELETE (d->command_list, node);
+          CDL_PREPEND (d->ready_list, node);
+          basic_command_scheduler (d);
+          POCL_UNLOCK (d->cq_lock);
+        }
+      return;
+    }
+}
+
+void
+pocl_basic_broadcast (cl_event event)
+{
+  pocl_broadcast (event);
+}
+
+void
+pocl_basic_compile_kernel (_cl_command_node *cmd, cl_kernel kernel, cl_device_id device)
+{
+  if (cmd != NULL && cmd->type == CL_COMMAND_NDRANGE_KERNEL)
+    pocl_check_dlhandle_cache (cmd, 0);
 }
