@@ -36,9 +36,9 @@
 typedef struct pool_thread_data thread_data;
 
 /* Initializes scheduler. Must be called before any kernel enqueue */
-void pthread_scheduler_init (size_t num_worker_threads, cl_device_id device);
+void pthread_scheduler_init (cl_device_id device);
 
-void pthread_scheduler_uinit ();
+void pthread_scheduler_uninit ();
 
 /* Gives ready-to-execute command for scheduler */
 void pthread_scheduler_push_command (_cl_command_node *cmd);
