@@ -22,9 +22,12 @@
 */
 #include "tce_common.h"
 #include "pocl_util.h"
+#include "pocl_cache.h"
+#include "pocl_llvm.h"
+#include "utlist.h"
+#include "common.h"
 
 #include "config.h"
-#include "install-paths.h"
 #include "pocl_runtime_config.h"
 #include "pocl_hash.h"
 
@@ -51,22 +54,23 @@
 #include <GlobalScope.hh>
 #include <Environment.hh>
 
-#include "pocl_cache.h"
-
 using namespace TTAMachine;
 
-#define COMMAND_LENGTH 256
-#define WORKGROUP_STRING_LENGTH 1024
-#define max(a,b) (((a) > (b)) ? (a) : (b))
+#include <algorithm>
 
-#define ALIGNMENT (max(ALIGNOF_FLOAT16, ALIGNOF_DOUBLE16))
+#define ALIGNMENT MAX_EXTENDED_ALIGNMENT
 
 //#define DEBUG_TTA_DRIVER
 
 TCEDevice::TCEDevice(cl_device_id dev, const char* adfName) :
   local_as(NULL), global_as(NULL), private_as(NULL), machine_file(adfName), parent(dev),
-  currentProgram(NULL), curKernelAddr(0), curKernel(NULL), globalCycleCount(0) {
+  currentProgram(NULL), curKernelAddr(0), curKernel(NULL), globalCycleCount(0),
+  ready_list(NULL), command_list(NULL) {
   parent->data = this;
+  pthread_mutex_init (&cq_lock, NULL);
+  POCL_INIT_LOCK(tce_compile_lock);
+  dev->address_bits = 32;
+  dev->autolocals_to_args = 1;
 #if defined(WORDS_BIGENDIAN) && WORDS_BIGENDIAN == 1
   needsByteSwap = false;
 #else
@@ -120,12 +124,8 @@ TCEDevice::findDataMemoryAddresses() {
   assert (prog != NULL);
 
   const TTAProgram::GlobalScope& globalScope = prog->globalScopeConst();
-  
-  try {
-    commandQueueAddr = globalScope.dataLabel("kernel_command").address().location();
-  } catch (const KeyNotFound& e) {
-    POCL_ABORT ("Could not find the shared data structures from the device binary.");
-  }    
+
+  commandQueueAddr = global_as->start() + TTA_UNALLOCATED_GLOBAL_SPACE;
 }
 
 void
@@ -193,10 +193,13 @@ TCEDevice::initMemoryManagement(const TTAMachine::Machine& mach) {
   init_mem_region 
     (&local_mem, (memory_address_t)local_as->start(), parent->local_mem_size);
   init_mem_region 
-    (&global_mem, (memory_address_t)global_as->start() + TTA_UNALLOCATED_GLOBAL_SPACE, 
+    (&global_mem, (memory_address_t)global_as->start() + TTA_UNALLOCATED_GLOBAL_SPACE + sizeof(__kernel_exec_cmd),
      parent->global_mem_size);
 }
 
+#define SUBST(x) "  -DKERNEL_EXE_CMD_OFFSET=" # x
+#define OFFSET_ARG(c) SUBST(c)
+
 TCEString
 TCEDevice::tceccCommandLine
 (_cl_command_run *run_cmd, const TCEString& inputSrc, 
@@ -218,15 +221,17 @@ TCEDevice::tceccCommandLine
     }
   else 
     {
-      deviceMainSrc = TCEString(PKGDATADIR) + "/" + mainC;
+      deviceMainSrc = TCEString(POCL_INSTALL_PRIVATE_DATADIR) + "/" + mainC;
       assert(access(deviceMainSrc.c_str(), R_OK) == 0);
-      poclIncludePathSwitch = " -I " PKGDATADIR "/include";
+      poclIncludePathSwitch = " -I " POCL_INSTALL_PRIVATE_DATADIR "/include";
     }
 
   TCEString extraFlags = extraParams;
   if (isMultiCoreMachine())
     extraFlags += " -ldthread -lsync-lu -llockunit";
 
+  extraFlags += OFFSET_ARG(TTA_UNALLOCATED_GLOBAL_SPACE);
+
   TCEString tempDir = run_cmd->tmp_dir;
 
   std::string kernelObjSrc = "";
@@ -237,10 +242,6 @@ TCEDevice::tceccCommandLine
     extraFlags += " " + 
       TCEString(pocl_get_string_option("POCL_TCECC_EXTRA_FLAGS", ""));
 
-  std::string userProgramBuildOptions;
-  if (run_cmd->kernel->program->compiler_options != NULL)
-    userProgramBuildOptions = run_cmd->kernel->program->compiler_options;
-
   std::string kernelMdSymbolName = "_";
   kernelMdSymbolName += run_cmd->kernel->name;
   kernelMdSymbolName += "_md";
@@ -250,7 +251,7 @@ TCEDevice::tceccCommandLine
      use case when producing the kernel capture scripts. */
   TCEString cmdLine;
   cmdLine << "tcecc -llwpr " + poclIncludePathSwitch + " " + deviceMainSrc + " " + 
-    userProgramBuildOptions + " " + kernelObjSrc + " " + inputSrc + 
+    " " + kernelObjSrc + " " + inputSrc +
     " -k " + kernelMdSymbolName +
     " -g -O3 --emit-llvm -o " + programBcFile + " " + extraFlags + ";";
 
@@ -313,51 +314,62 @@ pocl_tce_malloc (void *device_data, cl_mem_flags flags,
 }
 
 cl_int
-pocl_tce_alloc_mem_obj (cl_device_id device, cl_mem mem_obj)
+pocl_tce_alloc_mem_obj (cl_device_id device, cl_mem mem_obj, void* host_ptr)
 {
   void *b = NULL;
   cl_int flags = mem_obj->flags;
+  unsigned i;
 
-  /* if memory for this global memory is not yet allocated -> do it */
-  if (mem_obj->device_ptrs[device->global_mem_id].mem_ptr == NULL)
+  /* check if some driver has already allocated memory for this mem_obj 
+     in our global address space, and use that*/
+  for (i = 0; i < mem_obj->context->num_devices; ++i)
     {
-      b = pocl_tce_malloc
-        (device->data, flags, mem_obj->size, mem_obj->mem_host_ptr);
-      if (b == NULL) return CL_MEM_OBJECT_ALLOCATION_FAILURE;
-      mem_obj->device_ptrs[device->global_mem_id].mem_ptr = b;
-      mem_obj->device_ptrs[device->global_mem_id].global_mem_id = 
-        device->global_mem_id;
+      if (!mem_obj->device_ptrs[i].available)
+        continue;
+
+      if (mem_obj->device_ptrs[i].global_mem_id == device->global_mem_id
+          && mem_obj->device_ptrs[i].mem_ptr != NULL)
+        {
+          mem_obj->device_ptrs[device->dev_id].mem_ptr =
+            mem_obj->device_ptrs[i].mem_ptr;
+
+          return CL_SUCCESS;
+        }
     }
-  /* copy already allocated global mem info to devices own slot */
-  mem_obj->device_ptrs[device->dev_id] = 
-    mem_obj->device_ptrs[device->global_mem_id];
-    
+
+  b = pocl_tce_malloc
+    (device->data, flags, mem_obj->size, host_ptr);
+  if (b == NULL) return CL_MEM_OBJECT_ALLOCATION_FAILURE;
+
+  mem_obj->device_ptrs[device->dev_id].mem_ptr = b;
+
   return CL_SUCCESS;
 
 }
 
 void
 pocl_tce_write (void *data, const void *host_ptr, void *device_ptr, 
-                size_t /*offset*/, size_t cb)
+                size_t offset, size_t cb)
 {
   TCEDevice *d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
 #ifdef DEBUG_TTA_DRIVER
-  printf("host: write %x %x %u\n", host_ptr, chunk->start_address, cb);
+  printf("host: write %x %x %u\n", host_ptr, chunk->start_address + offset, cb);
 #endif
-  d->copyHostToDevice(host_ptr, chunk->start_address, cb);
+  d->copyHostToDevice(host_ptr, chunk->start_address + offset, cb);
 }
 
 void
 pocl_tce_read (void *data, void *host_ptr, const void *device_ptr, 
-               size_t /*offset*/, size_t cb)
+               size_t offset, size_t cb)
 {
   TCEDevice* d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
 #ifdef DEBUG_TTA_DRIVER
-  printf("host: read to %x (host) from %d (device) %u\n", host_ptr, chunk->start_address, cb);
+  printf("host: read to %x (host) from %d (device) %u\n", host_ptr,
+         chunk->start_address + offset, cb);
 #endif
-  d->copyDeviceToHost(chunk->start_address, host_ptr, cb);
+  d->copyDeviceToHost(chunk->start_address + offset, host_ptr, cb);
 }
 
 void *
@@ -385,52 +397,89 @@ pocl_tce_free (cl_device_id device, cl_mem mem_obj)
   free_chunk ((chunk_info_t*) ptr);
 }
 
-void 
-pocl_tce_run 
-(void *data, 
- _cl_command_node* cmd) {
+void
+pocl_tce_compile_kernel(_cl_command_node *cmd,
+                        cl_kernel kernel, cl_device_id device)
+{
+  if (cmd->type != CL_COMMAND_NDRANGE_KERNEL)
+    return;
 
+  void* data = cmd->device->data;
   TCEDevice *d = (TCEDevice*)data;
-  int error;
-  char bytecode[POCL_FILENAME_LENGTH];
-  uint32_t kernelAddr;
-  unsigned i;
 
-  assert (data != NULL);
+  if (!kernel)
+    kernel = cmd->command.run.kernel;
+  if (!device)
+    device = cmd->device;
+
+  POCL_LOCK(d->tce_compile_lock);
+  int error = pocl_llvm_generate_workgroup_function(device, kernel,
+      cmd->command.run.local_x, cmd->command.run.local_y,
+      cmd->command.run.local_z);
+
+  if (error) {
+    POCL_UNLOCK(d->tce_compile_lock);
+    POCL_MSG_PRINT_GENERAL("TCE: pocl_llvm_generate_workgroup_function()"
+                           " failed for kernel %s\n", kernel->name);
+    assert(error == 0);
+  }
+
+  char bytecode[POCL_FILENAME_LENGTH];
+
+  assert(d != NULL);
+  assert(cmd->command.run.kernel);
+  assert(cmd->command.run.tmp_dir);
 
   if (d->isNewKernel(&(cmd->command.run))) {
     std::string assemblyFileName(cmd->command.run.tmp_dir);
     assemblyFileName += "/parallel.tpef";
-    
-    std::string kernelMdSymbolName = "_";
-    kernelMdSymbolName += cmd->command.run.kernel->name;
-    kernelMdSymbolName += "_md";
-    
-    std::string userProgramBuildOptions;
-    if (cmd->command.run.kernel->program->compiler_options != NULL)
-      userProgramBuildOptions = cmd->command.run.kernel->program->compiler_options;
-    
+
     if (access (assemblyFileName.c_str(), F_OK) != 0)
       {
         error = snprintf (bytecode, POCL_FILENAME_LENGTH,
                           "%s%s", cmd->command.run.tmp_dir, POCL_PARALLEL_BC_FILENAME);
-        TCEString buildCmd = 
+        TCEString buildCmd =
           d->tceccCommandLine(&cmd->command.run, bytecode, assemblyFileName);
-        
+
 #ifdef DEBUG_TTA_DRIVER
       std::cerr << "CMD: " << buildCmd << std::endl;
 #endif
       error = system(buildCmd.c_str());
       if (error != 0)
-        POCL_ABORT("Error while running tcecc.");
+        POCL_ABORT("Error while running tcecc.\n");
       }
-    
+  }
+
+  POCL_UNLOCK(d->tce_compile_lock);
+}
+
+void
+pocl_tce_run(void *data, _cl_command_node* cmd)
+{
+  assert(cmd->type == CL_COMMAND_NDRANGE_KERNEL);
+
+  TCEDevice *d = (TCEDevice*)data;
+  uint32_t kernelAddr;
+  unsigned i;
+
+  assert(d != NULL);
+  assert(cmd->command.run.kernel);
+  assert(cmd->command.run.tmp_dir);
+
+  if (d->isNewKernel(&(cmd->command.run))) {
+    std::string assemblyFileName(cmd->command.run.tmp_dir);
+    assemblyFileName += "/parallel.tpef";
+
+    std::string kernelMdSymbolName = "_";
+    kernelMdSymbolName += cmd->command.run.kernel->name;
+    kernelMdSymbolName += "_md";
+
     try {
       d->loadProgramToDevice(assemblyFileName);
       d->restartProgram();
     } catch (Exception &e) {
       std::cerr << "error: " << e.errorMessage() << std::endl;
-      POCL_ABORT("error: Failed to load program to the TTA.");
+      POCL_ABORT("error: Failed to load program to the TTA.\n");
     }
 
     const TTAProgram::Program* prog = d->currentProgram;
@@ -441,7 +490,7 @@ pocl_tce_run
     try {
       kernelAddr = globalScope.dataLabel(kernelMdSymbolName).address().location();
     } catch (const KeyNotFound& e) {
-      POCL_ABORT ("Could not find the shared data structures from the device binary.");
+      POCL_ABORT("Could not find the shared data structures from the device binary.\n");
     }
     // cache the currently device loaded kernel info 
     d->updateCurrentKernel(&(cmd->command.run), kernelAddr);
@@ -569,6 +618,7 @@ pocl_tce_run
              d->readWordFromDevice(d->commandQueueAddr));
       sleep(1);
 #endif
+  usleep(20000);
   } while (d->readWordFromDevice(d->commandQueueAddr) != POCL_KST_FINISHED);
 
 #ifdef DEBUG_TTA_DRIVER
@@ -605,7 +655,7 @@ pocl_tce_map_mem (void *data, void *buf_ptr,
     {
         if (posix_memalign (&target, ALIGNMENT, size) != 0)
         {
-            POCL_ABORT ("Could not allocate memory.");
+            POCL_ABORT("Could not allocate memory.\n");
         }
     }
 
@@ -655,32 +705,47 @@ pocl_tce_init_build(void *data)
   return include_switch;
 }
 
-void 
-pocl_tce_build_hash (void *data, SHA1_CTX *build_hash)
+char *
+pocl_tce_build_hash (cl_device_id device)
 {
-  TCEDevice *tce_dev = (TCEDevice*)data;
+  TCEDevice *tce_dev = (TCEDevice*)device->data;
   FILE* adf_file = fopen (tce_dev->machine_file.c_str(), "r");
   size_t size;
   uint8_t* adf_data = 0;
   const char *extra_flags = NULL;
-  size_t ef_size;
 
   fseek (adf_file, 0 , SEEK_END);
   size = ftell (adf_file);
   fseek (adf_file, 0, SEEK_SET);
   adf_data = (uint8_t*)malloc (size);
   if (fread (adf_data, 1, size, adf_file) == 0)
-      POCL_ABORT ("Could not read ADF.");
-  
-  //TCEString machine_hash = tce->dev->hash();
-  pocl_SHA1_Update (build_hash, adf_data, size);
+      POCL_ABORT("Could not read ADF.\n");
+
+  SHA1_CTX ctx;
+  uint8_t bin_dig[SHA1_DIGEST_SIZE];
+  pocl_SHA1_Init(&ctx);
+  pocl_SHA1_Update(&ctx, adf_data, size);
+  pocl_SHA1_Final(&ctx, bin_dig);
+
+  char *result = (char *)calloc(1000, sizeof(char));
+  strcpy(result, "tce-");
+  char *temp = result + 4;
+  unsigned i;
+  for (i=0; i < SHA1_DIGEST_SIZE; i++)
+    {
+      *temp++ = (bin_dig[i] & 0x0F) + 65;
+      *temp++ = ((bin_dig[i] & 0xF0) >> 4) + 65;
+    }
+  *temp++ = '_';
+  *temp = 0;
 
   if (pocl_is_option_set("POCL_TCECC_EXTRA_FLAGS"))
     {
       extra_flags = pocl_get_string_option("POCL_TCECC_EXTRA_FLAGS", "");
-      ef_size = strlen (extra_flags);
-      pocl_SHA1_Update (build_hash, (uint8_t*)extra_flags, ef_size);
+      strncpy(temp, extra_flags, (1000-(temp-result)) );
     }
+
+  return result;
 }
 
 void
@@ -785,10 +850,99 @@ pocl_tce_read_rect (void *data,
   for (k = 0; k < region[2]; ++k)
     for (j = 0; j < region[1]; ++j)
       {
-        char const *__restrict h_ptr = adjusted_host_ptr + host_row_pitch * j 
+        char *__restrict__ h_ptr = adjusted_host_ptr + host_row_pitch * j 
           + host_slice_pitch * k;       
         size_t offset = base_offset + buffer_row_pitch * j 
           + buffer_slice_pitch * k;
-        pocl_tce_write (data, h_ptr, device_ptr, offset, region[0]);
+        pocl_tce_read (data, h_ptr, device_ptr, offset, region[0]);
       }
 }
+
+static void tce_command_scheduler (TCEDevice *d) 
+{
+  _cl_command_node *node;
+  
+  /* execute commands from ready list */
+  while ((node = d->ready_list))
+    {
+      assert (pocl_command_is_ready(node->event));
+      CDL_DELETE (d->ready_list, node);
+      POCL_UNLOCK(d->cq_lock);
+      assert (node->event->status == CL_SUBMITTED);
+      if (node->type == CL_COMMAND_NDRANGE_KERNEL)
+        pocl_tce_compile_kernel(node, NULL, NULL);
+      pocl_exec_command(node);
+      POCL_LOCK(d->cq_lock);
+    }
+    
+  return;
+}
+
+void
+pocl_tce_submit (_cl_command_node *node, cl_command_queue /*cq*/)
+{
+  TCEDevice *d = (TCEDevice*)node->device->data;
+
+  node->ready = 1;
+  POCL_LOCK(d->cq_lock);
+  pocl_command_push(node, &d->ready_list, &d->command_list);
+  POCL_UNLOCK_OBJ(node->event);
+
+  tce_command_scheduler (d);
+  POCL_UNLOCK(d->cq_lock);
+
+  return;
+}
+
+void pocl_tce_flush (cl_device_id device, cl_command_queue /*cq*/)
+{
+  TCEDevice *d = (TCEDevice*)device->data;
+
+  POCL_LOCK (d->cq_lock);
+  tce_command_scheduler (d);
+  POCL_UNLOCK (d->cq_lock);
+}
+
+
+void
+pocl_tce_join(cl_device_id device, cl_command_queue /*cq*/)
+{
+  TCEDevice *d = (TCEDevice*)device->data;
+
+  POCL_LOCK (d->cq_lock);
+  tce_command_scheduler (d);
+  POCL_UNLOCK (d->cq_lock);
+
+  return;
+}
+
+void
+pocl_tce_notify (cl_device_id device, cl_event event, cl_event finished)
+{
+  TCEDevice *d = (TCEDevice*)device->data;
+  _cl_command_node * volatile node = event->command;
+
+  if (finished->status < CL_COMPLETE) {
+    POCL_UPDATE_EVENT_FAILED(event);
+    return;
+  }
+
+  if (!node->ready)
+    return;
+
+  if (pocl_command_is_ready(event)) {
+    if (event->status == CL_QUEUED) {
+      POCL_UPDATE_EVENT_SUBMITTED(event);
+      POCL_LOCK(d->cq_lock);
+      CDL_DELETE(d->command_list, node);
+      CDL_PREPEND(d->ready_list, node);
+      POCL_UNLOCK(d->cq_lock);
+    }
+  }
+}
+
+void
+pocl_tce_broadcast (cl_event event)
+{
+  pocl_broadcast (event);
+}
