@@ -64,6 +64,8 @@
 #include "tce_common.h"
 #include "devices.h"
 
+#define DEFAULT_WG_SIZE 8192
+
 using namespace TTAMachine;
 
 static void *pocl_ttasim_thread (void *p);
@@ -76,6 +78,7 @@ pocl_ttasim_init_device_ops(struct pocl_device_ops *ops)
   ops->probe = pocl_ttasim_probe;
   ops->init_device_infos = pocl_ttasim_init_device_infos;
   ops->uninit = pocl_ttasim_uninit;
+  ops->reinit = NULL;
   ops->init = pocl_ttasim_init;
   ops->alloc_mem_obj = pocl_tce_alloc_mem_obj;
   ops->create_sub_buffer = pocl_tce_create_sub_buffer;
@@ -108,8 +111,17 @@ pocl_ttasim_init_device_infos(unsigned j, struct _cl_device_id* dev)
   dev->type = CL_DEVICE_TYPE_GPU;
   dev->max_compute_units = 1;
   dev->max_work_item_dimensions = 3;
-  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1] =
-	  dev->max_work_item_sizes[2] = dev->max_work_group_size = 8192;
+
+  int max_wg
+      = pocl_get_int_option("POCL_MAX_WORK_GROUP_SIZE", DEFAULT_WG_SIZE);
+  assert(max_wg > 0);
+  max_wg = std::min(max_wg, DEFAULT_WG_SIZE);
+  if (max_wg < 0)
+    max_wg = DEFAULT_WG_SIZE;
+
+  dev->max_work_item_sizes[0] = dev->max_work_item_sizes[1]
+      = dev->max_work_item_sizes[2] = dev->max_work_group_size = max_wg;
+
   dev->preferred_wg_size_multiple = 8;
   dev->preferred_vector_width_char = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_CHAR;
   dev->preferred_vector_width_short = POCL_DEVICES_PREFERRED_VECTOR_WIDTH_SHORT;
@@ -188,7 +200,7 @@ public:
     const char *adf = strrchr(adfName, '/');
     if (adf != NULL) adf++;
     if (snprintf (dev_name, 256, "ttasim-%s", adf) < 0)
-      POCL_ABORT("Unable to generate the device name string.");
+      POCL_ABORT("Unable to generate the device name string.\n");
     dev->long_name = strdup(dev_name);  
     ++device_count;
 
@@ -490,7 +502,7 @@ pocl_ttasim_thread (void *p)
 
         if (d->simulator.hadRuntimeError()) {
             d->simulatorCLI.run();
-            POCL_ABORT("Runtime error in a ttasim device.");
+            POCL_ABORT("Runtime error in a ttasim device.\n");
         }
     } while (false);
   } while (true);
@@ -541,10 +553,11 @@ pocl_ttasim_init (unsigned j, cl_device_id device, const char* parameters)
   return CL_SUCCESS;
 }
 
-void
+cl_int
 pocl_ttasim_uninit (cl_device_id device)
 {
   delete (TTASimDevice*)device->data;
+  return CL_SUCCESS;
 }
 
 
