@@ -21,16 +21,18 @@
    THE SOFTWARE.
 */
 
+#include <errno.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 
 #include "config.h"
-#ifdef POCL_BUILT_WITH_CMAKE
 #include "pocl_build_timestamp.h"
-#endif
+
+#ifdef OCS_AVAILABLE
 #include "kernellib_hash.h"
+#endif
 
 #include "pocl_hash.h"
 #include "pocl_cache.h"
@@ -49,7 +51,19 @@
 #define POCL_PROGRAM_BC_FILENAME "/program.bc"
 
 static char cache_topdir[POCL_FILENAME_LENGTH];
+static char tempfile_pattern[POCL_FILENAME_LENGTH];
+static char tempdir_pattern[POCL_FILENAME_LENGTH];
 static int cache_topdir_initialized = 0;
+static int use_kernel_cache = 0;
+
+/* sanity check on SHA1 digest emptiness */
+static unsigned buildhash_is_valid(cl_program   program, unsigned     device_i)
+{
+  unsigned i, sum = 0;
+  for(i=0; i<sizeof(SHA1_digest_t); i++)
+    sum += program->build_hash[device_i][i];
+  return sum;
+}
 
 int pocl_cl_device_to_index(cl_program   program,
                             cl_device_id device) {
@@ -62,15 +76,15 @@ int pocl_cl_device_to_index(cl_program   program,
     return -1;
 }
 
-static void program_device_dir(char*        path,
-                              cl_program   program,
-                              unsigned     device_i,
-                              const char*        append_path) {
+static void program_device_dir(char *path,
+                               cl_program program,
+                               unsigned device_i,
+                               const char* append_path)
+{
     assert(path);
     assert(program);
     assert(device_i < program->num_devices);
-    /* sanity check on SHA1 digest emptiness */
-    assert(program->build_hash[device_i][0] > 0);
+    assert(buildhash_is_valid(program, device_i));
 
     int bytes_written = snprintf(path, POCL_FILENAME_LENGTH,
                                  "%s/%s%s", cache_topdir,
@@ -79,6 +93,12 @@ static void program_device_dir(char*        path,
     assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
 }
 
+void pocl_cache_program_path(char*        path,
+                             cl_program   program,
+                             unsigned     device_i)
+{
+  program_device_dir (path, program, device_i, "");
+}
 
 // required in llvm API
 void pocl_cache_program_bc_path(char*        program_bc_path,
@@ -88,9 +108,14 @@ void pocl_cache_program_bc_path(char*        program_bc_path,
                        device_i, POCL_PROGRAM_BC_FILENAME);
 }
 
-static void pocl_cache_kernel_cachedir_path(char* kernel_cachedir_path, cl_program   program,
-                                 unsigned device_i, cl_kernel kernel, char* append_str,
-                                 size_t local_x, size_t local_y, size_t local_z)
+void pocl_cache_kernel_cachedir_path (char* kernel_cachedir_path,
+                                             cl_program program,
+                                             unsigned device_i,
+                                             cl_kernel kernel,
+                                             char* append_str,
+                                             size_t local_x,
+                                             size_t local_y,
+                                             size_t local_z)
 {
   int bytes_written;
   char tempstring[POCL_FILENAME_LENGTH];
@@ -113,6 +138,16 @@ static void pocl_cache_kernel_cachedir_path(char* kernel_cachedir_path, cl_progr
 
 }
 
+void pocl_cache_kernel_cachedir(char* kernel_cachedir_path, cl_program   program,
+                                unsigned device_i, cl_kernel kernel)
+{
+  int bytes_written;
+  char tempstring[POCL_FILENAME_LENGTH];
+  bytes_written = snprintf(tempstring, POCL_FILENAME_LENGTH,
+                           "/%s", kernel->name);
+  assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
+  program_device_dir(kernel_cachedir_path, program, device_i, tempstring);
+}
 
 
 // required in llvm API
@@ -156,86 +191,62 @@ void pocl_cache_final_binary_path(char* final_binary_path, cl_program program,
 }
 
 /******************************************************************************/
-
-static void* acquire_program_lock(cl_program program,
-                                  unsigned device_i,
-                                  const char* lock_type,
-                                  int shared) {
-    char lock_path[POCL_FILENAME_LENGTH];
-    program_device_dir(lock_path, program, device_i, lock_type);
-
-    return acquire_lock(lock_path, shared);
-}
-
-// EXCLUSIVE writer lock
-void* pocl_cache_acquire_writer_lock_i(cl_program program,
-                                       unsigned device_i) {
-    return acquire_program_lock(program, device_i, "_write", 0);
-}
-
-// SHARED reader lock (on clReleaseProgram, request EXCLUSIVE reader..)
-void* pocl_cache_acquire_reader_lock_i(cl_program program,
-                                       unsigned device_i) {
-    return acquire_program_lock(program, device_i, "_read", 1);
-}
-
-void pocl_cache_release_lock(void* lock) {
-    return release_lock(lock);
-}
-
-void* pocl_cache_acquire_writer_lock(cl_program program,
-                                     cl_device_id device) {
-    int index = pocl_cl_device_to_index(program, device);
-    assert(index >= 0);
-    return pocl_cache_acquire_writer_lock_i(program, (unsigned)index);
-}
-
-void* pocl_cache_acquire_reader_lock(cl_program program,
-                                     cl_device_id device) {
-    int index = pocl_cl_device_to_index(program, device);
-    assert(index >= 0);
-    return pocl_cache_acquire_reader_lock_i(program, (unsigned)index);
-}
-
 /******************************************************************************/
 
-void pocl_cache_mk_temp_name(char* path) {
-    assert(cache_topdir_initialized);
-#if defined(_MSC_VER) || defined(__MINGW32__)
-    char* tmp = _tempnam(cache_topdir, "pocl_");
-    assert(tmp);
-    int bytes_written = snprintf(path, POCL_FILENAME_LENGTH, "%s", tmp);
-    free(tmp);
-    assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
-#else
-    int bytes_written = snprintf(path, POCL_FILENAME_LENGTH,
-             "%s/temp_XXXXXX.cl", cache_topdir);
-    assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
-    /* using mkstemp() instead of tmpnam() has no real benefit
-     * here, as we have to pass the filename to llvm,
-     * but tmpnam() generates an annoying warning... */
-    int fd = mkstemps(path, 3);
-    assert(fd >= 0);
-    close(fd);
-#endif
+int
+pocl_cache_create_tempdir (char *path)
+{
+  return pocl_mk_tempdir (path, tempdir_pattern);
 }
 
-int pocl_cache_write_program_source(char *program_cl_path,
-                                    cl_program program) {
-    pocl_cache_mk_temp_name(program_cl_path);
-    return pocl_write_file(program_cl_path, program->source,
-                           strlen(program->source), 0, 0);
+int
+pocl_cache_tempname (char *path, const char *suffix, int *fd)
+{
+  assert (cache_topdir_initialized);
+  assert (path);
+
+  return pocl_mk_tempname (path, tempfile_pattern, suffix, fd);
 }
 
+int
+pocl_cache_write_program_source (char *program_cl_path, cl_program program)
+{
+  return pocl_write_tempfile (program_cl_path, tempfile_pattern,
+                              ".cl", program->source,
+                              strlen (program->source), NULL);
+}
+
+int
+pocl_cache_write_kernel_objfile (char *objfile_path,
+                                 const char *objfile_content,
+                                 size_t objfile_size)
+{
+  return pocl_write_tempfile (objfile_path, tempfile_pattern, ".so.o",
+                              objfile_content, objfile_size, NULL);
+}
+
+int
+pocl_cache_write_spirv (char *spirv_path,
+                        const char *spirv_content,
+                        size_t file_size)
+{
+  return pocl_write_tempfile (spirv_path, tempfile_pattern, ".spirv",
+                              spirv_content, file_size, NULL);
+}
+
+
 /******************************************************************************/
 
 int pocl_cache_update_program_last_access(cl_program program,
                                           unsigned device_i) {
-    char last_accessed_path[POCL_FILENAME_LENGTH];
-    program_device_dir(last_accessed_path, program,
-                       device_i, POCL_LAST_ACCESSED_FILENAME);
+  if (!use_kernel_cache)
+    return 0;
 
-    return pocl_touch_file(last_accessed_path);
+  char last_accessed_path[POCL_FILENAME_LENGTH];
+  program_device_dir (last_accessed_path, program, device_i,
+                      POCL_LAST_ACCESSED_FILENAME);
+
+  return pocl_touch_file (last_accessed_path);
 }
 
 /******************************************************************************/
@@ -275,7 +286,6 @@ int pocl_cache_write_descriptor(cl_program   program,
 
 /******************************************************************************/
 
-
 char* pocl_cache_read_buildlog(cl_program program,
                                unsigned device_i) {
     char buildlog_path[POCL_FILENAME_LENGTH];
@@ -285,7 +295,7 @@ char* pocl_cache_read_buildlog(cl_program program,
                        device_i, POCL_BUILDLOG_FILENAME);
 
     if (!pocl_exists(buildlog_path))
-      return strdup("");
+      return NULL;
 
     char* res=NULL;
     uint64_t filesize;
@@ -294,11 +304,13 @@ char* pocl_cache_read_buildlog(cl_program program,
     return res;
 }
 
-
 int pocl_cache_append_to_buildlog(cl_program  program,
                                   unsigned    device_i,
                                   const char *content,
                                   size_t      size) {
+    if (!buildhash_is_valid (program, device_i))
+      return -1;
+
     char buildlog_path[POCL_FILENAME_LENGTH];
     program_device_dir(buildlog_path, program,
                        device_i, POCL_BUILDLOG_FILENAME);
@@ -308,7 +320,7 @@ int pocl_cache_append_to_buildlog(cl_program  program,
 
 /******************************************************************************/
 
-
+#ifdef OCS_AVAILABLE
 int pocl_cache_write_kernel_parallel_bc(void*        bc,
                                         cl_program   program,
                                         unsigned     device_i,
@@ -331,31 +343,9 @@ int pocl_cache_write_kernel_parallel_bc(void*        bc,
     return pocl_write_module(bc, kernel_parallel_path, 0);
 }
 
-int pocl_cache_make_kernel_cachedir_path(char*        kernel_cachedir_path,
-                                         cl_program   program,
-                                         cl_device_id device,
-                                         cl_kernel    kernel,
-                                         size_t       local_x,
-                                         size_t       local_y,
-                                         size_t       local_z) {
-    int index = pocl_cl_device_to_index(program, device);
-    assert(index >= 0);
-
-    pocl_cache_kernel_cachedir_path(kernel_cachedir_path, program, index,
-                                     kernel, "", local_x, local_y, local_z);
-
-    return pocl_mkdir_p(kernel_cachedir_path);
-}
-
 
 /******************************************************************************/
 
-
-
-
-
-
-
 static inline void
 build_program_compute_hash(cl_program program,
                            unsigned   device_i,
@@ -373,12 +363,31 @@ build_program_compute_hash(cl_program program,
         assert(source_len > 0);
         pocl_SHA1_Update(&hash_ctx, (uint8_t*)preprocessed_source,
                          source_len);
-    } else     { /* Program was created with clCreateProgramWithBinary() */
-        assert(program->binaries[device_i]);
-        pocl_SHA1_Update(&hash_ctx,
-                         (uint8_t*) program->binaries[device_i],
-                         program->binary_sizes[device_i]);
-    }
+    } else if (program->pocl_binary_sizes[device_i] > 0) {
+      /* Program was created with clCreateProgramWithBinary() with
+	 a pocl binary */
+      assert(program->pocl_binaries[device_i]);
+      pocl_SHA1_Update(&hash_ctx,
+		       (uint8_t*) program->pocl_binaries[device_i],
+		       program->pocl_binary_sizes[device_i]);
+      }
+    else if (program->binary_sizes[device_i] > 0)
+      {
+        /* Program was created with clCreateProgramWithBinary() with an LLVM IR
+         * binary */
+        assert (program->binaries[device_i]);
+        pocl_SHA1_Update (&hash_ctx, (uint8_t *)program->binaries[device_i],
+                          program->binary_sizes[device_i]);
+      }
+    else
+      {
+        /* Program is linked from binaries, has no source or binary */
+        // assert(program->binary_type == CL_PROGRAM_BIN)
+        assert (preprocessed_source);
+        assert (source_len > 0);
+        pocl_SHA1_Update (&hash_ctx, (uint8_t *)preprocessed_source,
+                          source_len);
+      }
 
     if (program->compiler_options)
         pocl_SHA1_Update(&hash_ctx, (uint8_t*) program->compiler_options,
@@ -392,6 +401,10 @@ build_program_compute_hash(cl_program program,
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) wg_method, strlen(wg_method));
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) PACKAGE_VERSION,
                      strlen(PACKAGE_VERSION));
+#ifdef POCL_KCACHE_SALT
+    pocl_SHA1_Update(&hash_ctx, (uint8_t*) POCL_KCACHE_SALT,
+                     strlen(POCL_KCACHE_SALT));
+#endif
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) LLVM_VERSION,
                      strlen(LLVM_VERSION));
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) POCL_BUILD_TIMESTAMP,
@@ -400,8 +413,11 @@ build_program_compute_hash(cl_program program,
                      strlen(POCL_KERNELLIB_SHA1));
     /*devices may include their own information to hash */
     if (device->ops->build_hash)
-        device->ops->build_hash(device->data, &hash_ctx);
-
+      {
+        char *dev_hash = device->ops->build_hash(device);
+        pocl_SHA1_Update(&hash_ctx, (const uint8_t *)dev_hash, strlen(dev_hash));
+        free(dev_hash);
+      }
 
     uint8_t digest[SHA1_DIGEST_SIZE];
     pocl_SHA1_Final(&hash_ctx, digest);
@@ -417,6 +433,8 @@ build_program_compute_hash(cl_program program,
     program->build_hash[device_i][2] = '/';
 
 }
+#endif
+
 
 #ifdef ANDROID
 static
@@ -448,19 +466,23 @@ char* pocl_get_process_name()
 }
 #endif
 
-
 /******************************************************************************/
 
-void pocl_cache_init_topdir() {
+int
+pocl_cache_init_topdir ()
+{
+  if (cache_topdir_initialized)
+    return 0;
 
-    if (cache_topdir_initialized)
-        return;
+  use_kernel_cache
+      = pocl_get_bool_option ("POCL_KERNEL_CACHE", POCL_KERNEL_CACHE_DEFAULT);
 
-    const char *tmp_path = pocl_get_string_option("POCL_CACHE_DIR", NULL);
-    int needed;
+  const char *tmp_path = pocl_get_string_option ("POCL_CACHE_DIR", NULL);
+  int needed;
 
-    if (tmp_path && (pocl_exists(tmp_path))) {
-        needed = snprintf(cache_topdir, POCL_FILENAME_LENGTH, "%s", tmp_path);
+  if (tmp_path)
+    {
+      needed = snprintf (cache_topdir, POCL_FILENAME_LENGTH, "%s", tmp_path);
     } else     {
 #ifdef POCL_ANDROID
         char* process_name = pocl_get_process_name();
@@ -484,31 +506,76 @@ void pocl_cache_init_topdir() {
         // $HOME/.cache should be used."
         // http://standards.freedesktop.org/basedir-spec/latest/
         tmp_path = getenv("XDG_CACHE_HOME");
+        const char *p;
+        if (use_kernel_cache)
+          p = "pocl/kcache";
+        else
+          p = "pocl/uncached";
 
         if (tmp_path && tmp_path[0] != '\0') {
-            needed = snprintf(cache_topdir, POCL_FILENAME_LENGTH,
-                              "%s/pocl/kcache", tmp_path);
+            needed = snprintf (cache_topdir, POCL_FILENAME_LENGTH, "%s/%s",
+                               tmp_path, p);
         }
         else if ((tmp_path = getenv("HOME")) != NULL) {
-            needed = snprintf(cache_topdir, POCL_FILENAME_LENGTH,
-                              "%s/.cache/pocl/kcache", tmp_path);
+            needed = snprintf (cache_topdir, POCL_FILENAME_LENGTH,
+                               "%s/.cache/%s", tmp_path, p);
         }
         else {
-            needed = snprintf(cache_topdir, POCL_FILENAME_LENGTH,
-                              "/tmp/pocl/kcache");
+            needed
+                = snprintf (cache_topdir, POCL_FILENAME_LENGTH, "/tmp/%s", p);
         }
 #endif
     }
 
-    if (needed >= POCL_FILENAME_LENGTH) {
-        POCL_ABORT("pocl: cache path longer than maximum filename length");
+  if (needed >= POCL_FILENAME_LENGTH)
+    {
+      POCL_MSG_ERR ("pocl: cache path longer than maximum filename length\n");
+      return 1;
     }
 
     assert(strlen(cache_topdir) > 0);
+
     if (pocl_mkdir_p(cache_topdir))
-        POCL_ABORT("Could not create topdir for cache");
+      {
+        POCL_MSG_ERR (
+            "Could not create top directory (%s) for cache. \n\nNote: "
+            "if you have proper rights to create that directory, and still "
+            "get the error, then most likely pocl and the program you're "
+            "trying to run are linked to different versions of libstdc++ "
+            "library. \nThis is not a bug in pocl and there's nothing we "
+            "can do to fix it - you need both pocl and your program to be"
+            " compiled for your system. This is known to happen with "
+            "Luxmark benchmark binaries dowloaded from website; Luxmark "
+            "installed from your linux distribution's packages should "
+            "work.\n",
+            cache_topdir);
+        return 1;
+      }
+
+    strncpy (tempfile_pattern, cache_topdir, POCL_FILENAME_LENGTH);
+    size_t len = strlen (tempfile_pattern);
+    strncpy (tempfile_pattern + len, "/tempfile",
+             (POCL_FILENAME_LENGTH - len));
+    tempfile_pattern[POCL_FILENAME_LENGTH - 1] = 0;
+    assert (strlen (tempfile_pattern) < POCL_FILENAME_LENGTH);
+
+    int bytes_written;
+    if (use_kernel_cache)
+      {
+        bytes_written = snprintf (tempdir_pattern, POCL_FILENAME_LENGTH,
+                                  "%s/tempdir", cache_topdir);
+        assert (bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
+      }
+    else
+      {
+        bytes_written = snprintf (tempdir_pattern, POCL_FILENAME_LENGTH,
+                                  "%s/_UNCACHED", cache_topdir);
+        assert (bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
+      }
+
     cache_topdir_initialized = 1;
 
+    return 0;
 }
 
 /* Create the new program cachedir, invalidating the old program
@@ -526,78 +593,65 @@ pocl_cache_create_program_cachedir(cl_program program,
                                    size_t source_len,
                                    char* program_bc_path)
 {
-    const char *hash_source = NULL;
-    uint8_t old_build_hash[SHA1_DIGEST_SIZE] = {0};
-    size_t hs_len = 0;
-
     assert(cache_topdir_initialized);
 
-    if (program->source && preprocessed_source==NULL) {
-        hash_source = program->source;
-        hs_len = strlen(program->source);
-    } else {
-        hash_source = preprocessed_source;
-        hs_len = source_len;
-    }
+    if (use_kernel_cache)
+      {
+#ifdef OCS_AVAILABLE
+        const char *hash_source = NULL;
+        size_t hs_len = 0;
 
-    if (program->build_hash[device_i])
-        memcpy(old_build_hash, program->build_hash[device_i], SHA1_DIGEST_SIZE);
-
-    build_program_compute_hash(program, device_i, hash_source, hs_len);
-
-    /* if the old hash is nonzero and different, we must free the built binaries
-       before returning, so that they get loaded from the new location */
-    if (old_build_hash[0] && memcmp(old_build_hash, program->build_hash[device_i],
-            SHA1_DIGEST_SIZE))
-    {
-        if (program->binaries[device_i]) {
-            POCL_MEM_FREE(program->binaries[device_i]);
-            program->binary_sizes[device_i] = 0;
+        if (program->source && preprocessed_source==NULL) {
+            hash_source = program->source;
+            hs_len = strlen(program->source);
+        } else {
+            hash_source = preprocessed_source;
+            hs_len = source_len;
         }
-        pocl_free_llvm_irs(program, device_i);
-        pocl_cache_release_lock(program->read_locks[device_i]);
-        program->read_locks[device_i] = NULL;
-    }
 
-    program_device_dir(program_bc_path, program, device_i, "");
+        build_program_compute_hash(program, device_i, hash_source, hs_len);
 
-    if (pocl_mkdir_p(program_bc_path))
-        return 1;
+#else
+        assert(buildhash_is_valid(program, device_i));
+#endif
 
-    pocl_cache_program_bc_path(program_bc_path, program, device_i);
+        program_device_dir(program_bc_path, program, device_i, "");
 
-    program->read_locks[device_i] = pocl_cache_acquire_reader_lock_i(program, device_i);
-    assert(program->read_locks[device_i]);
+        if (pocl_mkdir_p(program_bc_path))
+            return 1;
+      }
+    else
+      {
+        /* if kernel cache is disabled, use a random dir. */
+        char random_dir[POCL_FILENAME_LENGTH];
+        if (pocl_cache_create_tempdir (random_dir))
+          return 1;
+        size_t s = strlen (cache_topdir) + 1;
+        assert (strlen (random_dir) == (s + 16));
+        memcpy (program->build_hash[device_i], random_dir + s, 16);
+      }
+
+    pocl_cache_program_bc_path (program_bc_path, program, device_i);
 
     return 0;
 }
 
 void pocl_cache_cleanup_cachedir(cl_program program) {
 
-    unsigned i;
+  /* only rm -rf if kernel cache is disabled */
+  if (use_kernel_cache)
+    return;
 
-    for (i = 0; i < program->num_devices; ++i)
-      pocl_cache_release_lock(program->read_locks[i]);
-    POCL_MEM_FREE(program->read_locks);
+  unsigned i;
 
-    if (!pocl_get_bool_option("POCL_KERNEL_CACHE", POCL_KERNEL_CACHE_DEFAULT)) {
+  for (i = 0; i < program->num_devices; i++)
+    {
+      if (!buildhash_is_valid (program, i))
+        continue;
 
-        for (i=0; i< program->num_devices; i++) {
-            if (program->build_hash[i][0] == 0)
-                continue;
-
-            void* lock = acquire_program_lock(program, i, "_read", 0);
-            if (!lock)
-              {
-                POCL_MSG_PRINT(" *** WARNING *** ", "",
-                "Could not get an exclusive lock to remove program cachedir");
-                continue;
-              }
-            char cachedir[POCL_FILENAME_LENGTH];
-            program_device_dir(cachedir, program, i, "");
-            pocl_rm_rf(cachedir);
-            release_lock(lock);
-        }
+      char cachedir[POCL_FILENAME_LENGTH];
+      program_device_dir (cachedir, program, i, "");
+      pocl_rm_rf (cachedir);
     }
 }
 
