@@ -63,27 +63,69 @@ typedef pthread_mutex_t pocl_lock_t;
 /* Generic functionality for handling different types of 
    OpenCL (host) objects. */
 
-#define POCL_LOCK(__LOCK__) pthread_mutex_lock (&(__LOCK__))
-#define POCL_UNLOCK(__LOCK__) pthread_mutex_unlock (&(__LOCK__))
-#define POCL_INIT_LOCK(__LOCK__) pthread_mutex_init (&(__LOCK__), NULL)
+#define POCL_LOCK(__LOCK__)                                                   \
+  do                                                                          \
+    {                                                                         \
+      int r = pthread_mutex_lock (&(__LOCK__));                               \
+      assert (r == 0);                                                        \
+    }                                                                         \
+  while (0)
+#define POCL_UNLOCK(__LOCK__)                                                 \
+  do                                                                          \
+    {                                                                         \
+      int r = pthread_mutex_unlock (&(__LOCK__));                             \
+      assert (r == 0);                                                        \
+    }                                                                         \
+  while (0)
+#define POCL_INIT_LOCK(__LOCK__)                                              \
+  do                                                                          \
+    {                                                                         \
+      int r = pthread_mutex_init (&(__LOCK__), NULL);                         \
+      assert (r == 0);                                                        \
+    }                                                                         \
+  while (0)
 /* We recycle OpenCL objects by not actually freeing them until the
    very end. Thus, the lock should not be destoryed at the refcount 0. */
-#define POCL_DESTROY_LOCK(__LOCK__) pthread_mutex_destroy (&(__LOCK__)) 
+#define POCL_DESTROY_LOCK(__LOCK__)                                           \
+  do                                                                          \
+    {                                                                         \
+      int r = pthread_mutex_destroy (&(__LOCK__));                            \
+      assert (r == 0);                                                        \
+    }                                                                         \
+  while (0)
 
-#define POCL_LOCK_OBJ(__OBJ__) POCL_LOCK((__OBJ__)->pocl_lock)
-#define POCL_UNLOCK_OBJ(__OBJ__) POCL_UNLOCK((__OBJ__)->pocl_lock)
+#define POCL_LOCK_OBJ(__OBJ__)                                                \
+  do                                                                          \
+    {                                                                         \
+      POCL_LOCK ((__OBJ__)->pocl_lock);                                       \
+      assert ((__OBJ__)->pocl_refcount > 0);                                  \
+    }                                                                         \
+  while (0)
+#define POCL_UNLOCK_OBJ(__OBJ__)                                              \
+  do                                                                          \
+    {                                                                         \
+      assert ((__OBJ__)->pocl_refcount >= 0);                                 \
+      POCL_UNLOCK ((__OBJ__)->pocl_lock);                                     \
+    }                                                                         \
+  while (0)
 
 #define POCL_RELEASE_OBJECT(__OBJ__, __NEW_REFCOUNT__)                  \
   do {                                                                  \
     POCL_LOCK_OBJ (__OBJ__);                                            \
     __NEW_REFCOUNT__ = --(__OBJ__)->pocl_refcount;                      \
-    assert((__OBJ__)->pocl_refcount >= 0);                              \
     POCL_UNLOCK_OBJ (__OBJ__);                                          \
   } while (0)
 
 #define POCL_RETAIN_OBJECT_UNLOCKED(__OBJ__)    \
     ++((__OBJ__)->pocl_refcount);
 
+#define POCL_RETAIN_OBJECT_REFCOUNT(__OBJ__, R) \
+  do {                                          \
+    POCL_LOCK_OBJ (__OBJ__);                    \
+    R = POCL_RETAIN_OBJECT_UNLOCKED (__OBJ__);  \
+    POCL_UNLOCK_OBJ (__OBJ__);                  \
+  } while (0)
+
 #define POCL_RETAIN_OBJECT(__OBJ__)             \
   do {                                          \
     POCL_LOCK_OBJ (__OBJ__);                    \
@@ -274,7 +316,9 @@ struct pocl_device_ops {
 
   /* /New driver api extension */
 
-  void (*uninit) (cl_device_id device);
+  cl_int (*uninit) (cl_device_id device);
+  cl_int (*reinit) (cl_device_id device);
+
   unsigned int (*probe) (struct pocl_device_ops *ops);
   /* Device initialization. Parameters:
    *  j : progressive index for the devices of the same type
@@ -454,7 +498,6 @@ struct _cl_device_id {
   size_t printf_buffer_size;
   char *short_name;
   char *long_name;
-  char *cache_dir_name;
 
   const char *vendor;
   const char *driver_version;
@@ -680,8 +723,6 @@ struct _cl_program {
   char main_build_log[640];
   /* Used to store the llvm IR of the build to save disk I/O. */
   void **llvm_irs;
-  /* Read locks */
-  void** read_locks;
   /* Use to store build status */
   cl_build_status build_status;
   /* Use to store binary type */
@@ -767,7 +808,6 @@ struct _cl_event {
 typedef struct _pocl_user_event_data
 {
   pthread_cond_t wakeup_cond;
-  pthread_mutex_t lock;
 } pocl_user_event_data;
 
 typedef struct _cl_sampler cl_sampler_t;
@@ -823,7 +863,7 @@ struct _cl_sampler {
     }                                                                         \
   while (0)
 
-#define POCL_UPDATE_EVENT_RUNNING(__event)                                    \
+#define POCL_UPDATE_EVENT_RUNNING_UNLOCKED(__event)                           \
   do                                                                          \
     {                                                                         \
       if (__event != NULL)                                                    \
@@ -845,21 +885,25 @@ struct _cl_sampler {
     }                                                                         \
   while (0)
 
+#define POCL_UPDATE_EVENT_RUNNING(__event)                                    \
+  POCL_LOCK_OBJ (__event);                                                    \
+  POCL_UPDATE_EVENT_RUNNING_UNLOCKED (__event);                               \
+  POCL_UNLOCK_OBJ (__event);
+
 #define POCL_UPDATE_EVENT_COMPLETE_INNER(__event, POST_EVENT)                 \
   do                                                                          \
     {                                                                         \
       if ((__event) != NULL)                                                  \
         {                                                                     \
           assert ((__event)->status == CL_RUNNING);                           \
+          POCL_LOCK_OBJ (__event);                                            \
           cl_command_queue __cq = (__event)->queue;                           \
-          assert ((__event)->status == CL_RUNNING);                           \
           if ((__cq)->device->ops->update_event)                              \
             (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
                                                CL_COMPLETE);                  \
           else                                                                \
             {                                                                 \
               pocl_mem_objs_cleanup (__event);                                \
-              POCL_LOCK_OBJ (__event);                                        \
               (__event)->status = CL_COMPLETE;                                \
               if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
                 {                                                             \
@@ -868,11 +912,13 @@ struct _cl_sampler {
                           (__cq)->device->data);                              \
                 }                                                             \
               POCL_UNLOCK_OBJ (__event);                                      \
-              (__cq)->device->ops->broadcast (__event);                       \
               pocl_update_command_queue (__event);                            \
+              (__cq)->device->ops->broadcast (__event);                       \
+              POCL_LOCK_OBJ (__event);                                        \
             }                                                                 \
           pocl_event_updated (__event, CL_COMPLETE);                          \
           POST_EVENT;                                                         \
+          POCL_UNLOCK_OBJ (__event);                                          \
           POname (clReleaseEvent) (__event);                                  \
         }                                                                     \
     }                                                                         \
@@ -899,7 +945,6 @@ struct _cl_sampler {
           else                                                                \
             {                                                                 \
               pocl_mem_objs_cleanup (__event);                                \
-              POCL_LOCK_OBJ (__event);                                        \
               if ((__event)->status > CL_COMPLETE)                            \
                 (__event)->status = CL_FAILED;                                \
               if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
@@ -909,11 +954,14 @@ struct _cl_sampler {
                           (__cq)->device->data);                              \
                 }                                                             \
               POCL_UNLOCK_OBJ (__event);                                      \
-              (__cq)->device->ops->broadcast (__event);                       \
               pocl_update_command_queue (__event);                            \
+              (__cq)->device->ops->broadcast (__event);                       \
+              POCL_LOCK_OBJ (__event);                                        \
             }                                                                 \
           pocl_event_updated (__event, CL_FAILED);                            \
+          POCL_UNLOCK_OBJ (__event);                                          \
           POname (clReleaseEvent) (__event);                                  \
+          POCL_LOCK_OBJ (__event);                                            \
         }                                                                     \
     }                                                                         \
   while (0)
@@ -923,6 +971,18 @@ struct _cl_sampler {
 #define min(a,b) (((a) < (b)) ? (a) : (b))
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
+#ifndef __APPLE__
+  #include <endian.h>
+#else
+  #include <libkern/OSByteOrder.h>
+  #define htole16(x) OSSwapHostToLittleInt16(x)
+  #define le16toh(x) OSSwapLittleToHostInt16(x)
+  #define htole32(x) OSSwapHostToLittleInt32(x)
+  #define le32toh(x) OSSwapLittleToHostInt32(x)
+  #define htole64(x) OSSwapHostToLittleInt64(x)
+  #define le64toh(x) OSSwapLittleToHostInt64(x)
+#endif
+
 #endif
 
 #endif /* POCL_CL_H */
