@@ -41,27 +41,31 @@ pocl_get_image_information (cl_channel_order  ch_order,
                             cl_int*           host_channels,
                             cl_int*           host_elem_size);
 
-extern cl_int
-pocl_check_device_supports_image(const cl_mem image,
-                                 const cl_command_queue command_queue);
+extern cl_int pocl_check_device_supports_image (
+    cl_device_id device, const cl_image_format *image_format,
+    const cl_image_desc *image_desc, cl_image_format *supported_image_formats,
+    cl_uint num_entries);
 
-extern cl_int
-pocl_write_image(cl_mem               image,
-                 cl_device_id         device_id,
-                 const size_t *       origin_, /*[3]*/
-                 const size_t *       region_, /*[3]*/
-                 size_t               host_row_pitch,
-                 size_t               host_slice_pitch, 
-                 const void *         ptr);
+void pocl_write_pixel_zero (void *data, const void *color_ptr, int order,
+                            int elem_size, int channel_type);
 
-extern cl_int         
-pocl_read_image(cl_mem               image,
-                cl_device_id         device,
-                const size_t *       origin, /*[3]*/
-                const size_t *       region, /*[3]*/
-                size_t               host_row_pitch,
-                size_t               host_slice_pitch, 
-                void *               ptr);
+cl_char4 convert_char4_sat (cl_float4 x);
+cl_char convert_char_sat (cl_float x);
+cl_char4 convert_char4_sat_int (cl_int4 x);
+cl_char convert_char_sat_int (cl_int x);
+cl_uchar4 convert_uchar4_sat (cl_float4 x);
+cl_uchar convert_uchar_sat (cl_float x);
+cl_uchar4 convert_uchar4_sat_int (cl_uint4 x);
+cl_uchar convert_uchar_sat_int (cl_uint x);
+
+cl_short4 convert_short4_sat (cl_float4 x);
+cl_short convert_short_sat (cl_float x);
+cl_short4 convert_short4_sat_int (cl_int4 x);
+cl_short convert_short_sat_int (cl_int x);
+cl_ushort4 convert_ushort4_sat (cl_float4 x);
+cl_ushort convert_ushort_sat (cl_float x);
+cl_ushort4 convert_ushort4_sat_int (cl_uint4 x);
+cl_ushort convert_ushort_sat_int (cl_uint x);
 
 #ifdef __GNUC__
 #pragma GCC visibility pop
