@@ -42,7 +42,7 @@ struct env_data
 static env_data *volatile env_cache = 0;
 static pocl_lock_t lock = POCL_LOCK_INITIALIZER;
 
-env_data* find_env (env_data* cache, const char* key)
+static env_data* find_env (env_data* cache, const char* key)
 {
   env_data* ed;
   char *value;
@@ -62,7 +62,7 @@ env_data* find_env (env_data* cache, const char* key)
       ed->env = strdup (key);
       ed->value = strdup (value);
       ed->next = NULL;
-      LL_PREPEND(env_cache, ed);
+      LL_APPEND(env_cache, ed);
       POCL_UNLOCK(lock);
       return ed;
     }
