@@ -37,9 +37,29 @@
 extern "C" {
 #endif
 
+#if defined(__GNUC__) || defined(__clang__)
+#define ATOMIC_INC(x) __sync_add_and_fetch(&x, 1)
+#else
+#error Need atomic_inc() builtin for this compiler
+#endif
+
 uint32_t byteswap_uint32_t (uint32_t word, char should_swap);
 float byteswap_float (float word, char should_swap);
 
+/* set rounding mode */
+void pocl_restore_rm (unsigned rm);
+/* get current rounding mode */
+unsigned pocl_save_rm ();
+/* set OpenCL's default (round to nearest) rounding mode */
+void pocl_set_default_rm ();
+
+/* sets the flush-denorms-to-zero flag on the CPU, if supported */
+void pocl_set_ftz (unsigned ftz);
+
+/* saves / restores cpu flags*/
+unsigned pocl_save_ftz ();
+void pocl_restore_ftz (unsigned ftz);
+
 /* Finds the next highest power of two of the given value. */
 size_t pocl_size_ceil2(size_t x);
 
@@ -59,6 +79,12 @@ void *pocl_aligned_malloc(size_t alignment, size_t size);
 void pocl_aligned_free(void* ptr);
 #endif
 
+/* locks / unlocks two events in order of their event-id.
+ * This avoids any potential deadlocks of threads should
+ * they try to lock events in opposite order. */
+void pocl_lock_events_inorder (cl_event ev1, cl_event ev2);
+void pocl_unlock_events_inorder (cl_event ev1, cl_event ev2);
+
 /* Function for creating events */
 cl_int pocl_create_event (cl_event *event, cl_command_queue command_queue,
                           cl_command_type command_type, int num_buffers,
@@ -116,6 +142,9 @@ pocl_command_is_ready(cl_event event)
 int
 pocl_update_command_queue (cl_event event);
 
+void pocl_cl_mem_inherit_flags (cl_mem mem, cl_mem from_buffer,
+                                cl_mem_flags flags);
+
 cl_int 
 pocl_update_mem_obj_sync (cl_command_queue cq, _cl_command_node *cmd, 
                           cl_mem mem, char operation);
@@ -124,29 +153,39 @@ void pocl_setup_context(cl_context context);
 
 /* Helpers for dealing with devices / subdevices */
 
-#define POCL_REAL_DEV(dev) (dev->parent_device ? dev->parent_device : dev)
-
+cl_device_id pocl_real_dev (const cl_device_id);
 cl_device_id * pocl_unique_device_list(const cl_device_id * in, cl_uint num, cl_uint *real);
 
-#define POCL_CHECK_DEV_IN_CMDQ                                               \
-  do                                                                         \
-    {                                                                        \
-      device = command_queue->device;                                        \
-      for (i = 0; i < command_queue->context->num_devices; ++i)              \
-        {                                                                    \
-          if (command_queue->context->devices[i] == POCL_REAL_DEV(device))   \
-            break;                                                           \
-        }                                                                    \
-      assert(i < command_queue->context->num_devices);                       \
-    }                                                                        \
+#define POCL_CHECK_DEV_IN_CMDQ                                                \
+  do                                                                          \
+    {                                                                         \
+      device = pocl_real_dev (command_queue->device);                         \
+      for (i = 0; i < command_queue->context->num_devices; ++i)               \
+        {                                                                     \
+          if (command_queue->context->devices[i] == device)                   \
+            break;                                                            \
+        }                                                                     \
+      assert (i < command_queue->context->num_devices);                       \
+    }                                                                         \
   while (0)
 
+int pocl_check_event_wait_list(cl_command_queue     command_queue,
+                               cl_uint              num_events_in_wait_list,
+                               const cl_event *     event_wait_list);
+
 const char*
 pocl_status_to_str (int status);
 
 const char *
 pocl_command_to_str (cl_command_type cmd);
 
+int
+pocl_run_command(char * const *args);
+
+uint16_t float_to_half (float value);
+
+float half_to_float (uint16_t value);
+
 #ifdef __cplusplus
 }
 #endif
@@ -188,6 +227,26 @@ pocl_command_to_str (cl_command_type cmd);
     }                                                                   \
   while (0)
 
+#define POCL_RETURN_GETINFO_STR_FREE(__STR__)                                 \
+  do                                                                          \
+    {                                                                         \
+      size_t const value_size = strlen (__STR__) + 1;                         \
+      if (param_value)                                                        \
+        {                                                                     \
+          if (param_value_size >= value_size)                                 \
+            memcpy (param_value, __STR__, value_size);                        \
+          POCL_MEM_FREE (__STR__);                                            \
+          if (param_value_size < value_size)                                  \
+            return CL_INVALID_VALUE;                                          \
+        }                                                                     \
+      else                                                                    \
+        POCL_MEM_FREE (__STR__);                                              \
+      if (param_value_size_ret)                                               \
+        *param_value_size_ret = value_size;                                   \
+      return CL_SUCCESS;                                                      \
+    }                                                                         \
+  while (0)
+
 #define POCL_RETURN_GETINFO(__TYPE__, __VALUE__)                        \
   do                                                                    \
     {                                                                   \
@@ -206,4 +265,9 @@ pocl_command_to_str (cl_command_type cmd);
     }                                                                   \
   while (0)
 
+#define HANDLE_IMAGE1D_BUFFER(mem)                                            \
+  mem = ((mem->is_image && (mem->type == CL_MEM_OBJECT_IMAGE1D_BUFFER))       \
+             ? mem->buffer                                                    \
+             : mem);
+
 #endif
