@@ -25,10 +25,16 @@
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wformat-security"
 
-#include <limits.h>
+#ifdef cl_khr_fp64
+#define LARGEST_FLOAT_TYPE double
+#define NAN __builtin_nan("1")
+#else
+#define LARGEST_FLOAT_TYPE float
+#define NAN __builtin_nanf("1")
+#endif
+
 #include <stdarg.h>
-#include <stdbool.h>
-#include <math.h>
+#include <limits.h>
 
 // We implement the OpenCL printf by calling the C99 printf. This is
 // not very efficient, but is easy to implement.
@@ -48,15 +54,13 @@ int snprintf(OCL_C_AS char* restrict str, size_t size,
 
 // Conversion flags
 typedef struct {
-  bool left;
-  bool plus;
-  bool space;
-  bool alt;
-  bool zero;
+  int left:1;
+  int plus:1;
+  int space:1;
+  int alt:1;
+  int zero:1;
 } flags_t;
 
-
-
 // Helper routines to output integers
 
 #define INT_CONV_char  "hh"
@@ -139,9 +143,9 @@ float __attribute__((overloadable)) vload_half(size_t offset,
       DEBUG_PRINTF(("[printf:floats:d=%d]\n", d));                      \
       if (d != 0) printf(comma);                                        \
       WIDTH val = (FLOAT_GET_##WIDTH((OCL_C_AS const WIDTH*)vals+d));   \
-      if (isnan (val))                                                  \
+      if (val != val)                                                   \
         val = NAN;                                                      \
-      printf(outfmt, (double)val);                                      \
+      printf (outfmt, (LARGEST_FLOAT_TYPE)val);                         \
     }                                                                   \
     DEBUG_PRINTF(("[printf:floats:done]\n"));                           \
   }
@@ -249,18 +253,18 @@ int __cl_printf(const OCL_CONSTANT_AS char* restrict format, ...)
         DEBUG_PRINTF(("[printf:arg]\n"));
         // Flags
         flags_t flags;
-        flags.left = false;
-        flags.plus = false;
-        flags.space = false;
-        flags.alt = false;
-        flags.zero = false;
+        flags.left = 0;
+        flags.plus = 0;
+        flags.space = 0;
+        flags.alt = 0;
+        flags.zero = 0;
         for (;;) {
           switch (ch) {
-          case '-': if (flags.left) goto error; flags.left = true; break;
-          case '+': if (flags.plus) goto error; flags.plus = true; break;
-          case ' ': if (flags.space) goto error; flags.space = true; break;
-          case '#': if (flags.alt) goto error; flags.alt = true; break;
-          case '0': if (flags.zero) goto error; flags.zero = true; break;
+          case '-': if (flags.left) goto error; flags.left = 1; break;
+          case '+': if (flags.plus) goto error; flags.plus = 1; break;
+          case ' ': if (flags.space) goto error; flags.space = 1; break;
+          case '#': if (flags.alt) goto error; flags.alt = 1; break;
+          case '0': if (flags.zero) goto error; flags.zero = 1; break;
           default: goto flags_done;
           }
           ch = *++format;
@@ -342,7 +346,8 @@ int __cl_printf(const OCL_CONSTANT_AS char* restrict format, ...)
         case 'u':
         case 'x':
         case 'X':
-          
+
+/* TODO: 3-size vector va-arg crashes LLVM when compiled with -O > 0 */
 #define CALL_PRINT_INTS(WIDTH, PROMOTED_WIDTH)                          \
           {                                                             \
             WIDTH##16 val;                                              \
@@ -350,7 +355,7 @@ int __cl_printf(const OCL_CONSTANT_AS char* restrict format, ...)
             default: __builtin_unreachable();                           \
             case 1: val.s0 = va_arg(ap, PROMOTED_WIDTH); break;         \
             case 2: val.s01 = va_arg(ap, WIDTH##2); break;              \
-            case 3: val.s012 = va_arg(ap, WIDTH##3); break;             \
+            case 3:                                                     \
             case 4: val.s0123 = va_arg(ap, WIDTH##4); break;            \
             case 8: val.lo = va_arg(ap, WIDTH##8); break;               \
             case 16: val = va_arg(ap, WIDTH##16); break;                \
@@ -384,7 +389,8 @@ int __cl_printf(const OCL_CONSTANT_AS char* restrict format, ...)
         case 'G':
         case 'a':
         case 'A':
-          
+
+/* TODO: 3-size vector va-arg crashes LLVM when compiled with -O > 0 */
 #define CALL_PRINT_FLOATS(WIDTH, PROMOTED_WIDTH)                        \
           {                                                             \
             WIDTH##16 val;                                              \
@@ -392,7 +398,7 @@ int __cl_printf(const OCL_CONSTANT_AS char* restrict format, ...)
             default: __builtin_unreachable();                           \
             case 1: val.s0 = va_arg(ap, PROMOTED_WIDTH); break;         \
             case 2: val.s01 = va_arg(ap, WIDTH##2); break;              \
-            case 3: val.s012 = va_arg(ap, WIDTH##3); break;             \
+            case 3:                                                     \
             case 4: val.s0123 = va_arg(ap, WIDTH##4); break;            \
             case 8: val.lo = va_arg(ap, WIDTH##8); break;               \
             case 16: val = va_arg(ap, WIDTH##16); break;                \
