@@ -50,16 +50,16 @@ using namespace pocl;
 
 namespace {
   class AutomaticLocals : public ModulePass {
-  
+
   public:
     static char ID;
     AutomaticLocals() : ModulePass(ID) {}
-    
+
     virtual void getAnalysisUsage(AnalysisUsage &AU) const;
     virtual bool runOnModule(Module &M);
 
   private:
-    Function *ProcessAutomaticLocals(Function *F);
+    Function *processAutomaticLocals(Function *F);
   };
 }
 
@@ -67,12 +67,16 @@ char AutomaticLocals::ID = 0;
 static RegisterPass<AutomaticLocals> X("automatic-locals",
 				      "Processes automatic locals");
 
+#if (LLVM_OLDER_THAN_3_7)
 void
 AutomaticLocals::getAnalysisUsage(AnalysisUsage &AU) const {
-#if (LLVM_OLDER_THAN_3_7)
   AU.addRequired<DataLayoutPass>();
-#endif
 }
+#else
+void
+AutomaticLocals::getAnalysisUsage(AnalysisUsage &) const {
+}
+#endif
 
 bool
 AutomaticLocals::runOnModule(Module &M)
@@ -85,23 +89,27 @@ AutomaticLocals::runOnModule(Module &M)
   FunctionMapping kernels;
 
   string ErrorInfo;
-
+  std::vector<Function*> NewFuncs;
   for (Module::iterator mi = M.begin(), me = M.end(); mi != me; ++mi) {
+    // This is to prevent recursion with llvm 3.9. The new kernels are
+    // recognized as kernelsToProcess.
+    if (find(NewFuncs.begin(), NewFuncs.end(), &*mi) != NewFuncs.end())
+      continue;
     if (!Workgroup::isKernelToProcess(*mi))
       continue;
-  
+
     Function *F = &*mi;
 
-    Function *new_kernel = ProcessAutomaticLocals(F);
+    Function *new_kernel = processAutomaticLocals(F);
     if (new_kernel != F)
       changed = true;
     kernels[F] = new_kernel;
+    NewFuncs.push_back(new_kernel);
   }
 
   if (changed)
     {
       regenerate_kernel_metadata(M, kernels);
-
       /* Delete the old kernels. */
       for (FunctionMapping::const_iterator i = kernels.begin(),
              e = kernels.end(); i != e; ++i) 
@@ -115,64 +123,85 @@ AutomaticLocals::runOnModule(Module &M)
   return changed;
 }
 
-Function *
-AutomaticLocals::ProcessAutomaticLocals(Function *F)
-{
-  Module *M = F->getParent();
-  
-  SmallVector<GlobalVariable *, 8> locals;
+// Recursively descend a Value's users and convert any constant expressions into
+// regular instructions.
+static void breakConstantExpressions(llvm::Value *Val, llvm::Function *Func) {
+  std::vector<llvm::Value *> Users(Val->user_begin(), Val->user_end());
+  for (auto *U : Users) {
+    if (auto *CE = llvm::dyn_cast<llvm::ConstantExpr>(U)) {
+      // First, make sure no users of this constant expression are themselves
+      // constant expressions.
+      breakConstantExpressions(U, Func);
 
-  SmallVector<Type *, 8> parameters;
+      // Convert this constant expression to an instruction.
+      llvm::Instruction *I = CE->getAsInstruction();
+      I->insertBefore(&*Func->begin()->begin());
+      CE->replaceAllUsesWith(I);
+      CE->destroyConstant();
+    }
+  }
+}
+
+Function *
+AutomaticLocals::processAutomaticLocals(Function *F) {
+
+  Module *M = F->getParent();
+
+  SmallVector<GlobalVariable *, 8> Locals;
+
+  SmallVector<Type *, 8> Parameters;
   for (Function::const_arg_iterator i = F->arg_begin(),
-         e = F->arg_end();
-       i != e; ++i)
-    parameters.push_back(i->getType());
-    
+         e = F->arg_end(); i != e; ++i)
+    Parameters.push_back(i->getType());
+
   for (Module::global_iterator i = M->global_begin(),
-         e = M->global_end();
-       i != e; ++i) {
-    std::string funcName = "";
-    funcName = F->getName().str();
-    if (is_automatic_local(funcName, *i)) {
-      locals.push_back(&*i);
+         e = M->global_end(); i != e; ++i) {
+    std::string FuncName = "";
+    FuncName = F->getName().str();
+    if (isAutomaticLocal(FuncName, *i)) {
+      Locals.push_back(&*i);
       // Add the parameters to the end of the function parameter list.
-      parameters.push_back(i->getType());
+      Parameters.push_back(i->getType());
+
+      // Replace any constant expression users with an equivalent instruction.
+      // Otherwise, the IR breaks when we replace the local with an argument.
+      breakConstantExpressions(&*i, F);
     }
   }
 
-  if (locals.empty()) {
+  if (Locals.empty()) {
     // This kernel fingerprint has not changed.
     return F;
   }
-  
+
   // Create the new function.
-  FunctionType *ft = FunctionType::get(F->getReturnType(),
-                                       parameters,
-                                       F->isVarArg());
-  Function *new_kernel = Function::Create(ft,
-                                          F->getLinkage(),
-                                          "",
-                                          M);
-  new_kernel->takeName(F);
-  
-  ValueToValueMapTy vv;
-  Function::arg_iterator j = new_kernel->arg_begin();
-  for (Function::const_arg_iterator i = F->arg_begin(),
-         e = F->arg_end();
+  FunctionType *FT =
+    FunctionType::get(F->getReturnType(), Parameters, F->isVarArg());
+  Function *NewKernel = Function::Create(FT, F->getLinkage(), "", M);
+  NewKernel->takeName(F);
+
+  ValueToValueMapTy VV;
+  Function::arg_iterator j = NewKernel->arg_begin();
+  for (Function::const_arg_iterator i = F->arg_begin(), e = F->arg_end();
        i != e; ++i) {
     j->setName(i->getName());
-    vv[&*i] = &*j;
+    VV[&*i] = &*j;
     ++j;
   }
-  
-  for (int i = 0; j != new_kernel->arg_end(); ++i, ++j) {
-    j->setName("_local" + Twine(i));
-    vv[locals[i]] = &*j;
-  }
-                                 
-  SmallVector<ReturnInst *, 1> ri;
-  CloneFunctionInto(new_kernel, F, vv, false, ri);
 
-  return new_kernel;
+  for (int i = 0; j != NewKernel->arg_end(); ++i, ++j) {
+    j->setName("_local" + Twine(i));
+    VV[Locals[i]] = &*j;
+  }
+
+  SmallVector<ReturnInst *, 1> RI;
+
+  // As of LLVM 5.0 we need to let CFI to make module level changes,
+  // otherwise there will be an assertion. The changes are likely
+  // additional debug info nodes added when cloning the function into
+  // the other.  For some reason it doesn't want to reuse the old ones.
+  CloneFunctionInto(NewKernel, F, VV, true, RI);
+
+  return NewKernel;
 }
 
