@@ -1,8 +1,8 @@
 // LLVM loop pass that adds required barriers to loops.
-// 
+//
 // Copyright (c) 2011 Universidad Rey Juan Carlos
 //               2012-2013 Pekka Jääskeläinen / Tampere University of Technology
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
@@ -54,38 +54,28 @@ namespace {
 char LoopBarriers::ID = 0;
 
 void
-LoopBarriers::getAnalysisUsage(AnalysisUsage &AU) const
-{
+LoopBarriers::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addRequired<DominatorTreeWrapperPass>();
-  AU.addPreserved<DominatorTreeWrapperPass>();
-
 }
 
 bool
-LoopBarriers::runOnLoop(Loop *L, LPPassManager &LPM)
-{
+LoopBarriers::runOnLoop(Loop *L, LPPassManager &LPM) {
   if (!Workgroup::isKernelToProcess(*L->getHeader()->getParent()))
     return false;
 
   if (!Workgroup::hasWorkgroupBarriers(*L->getHeader()->getParent()))
     return false;
 
-  DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
-
-  bool changed = ProcessLoop(L, LPM);
-
-  DT->verifyDomTree();
-
-  return changed;
+  return ProcessLoop(L, LPM);;
 }
 
-
 bool
-LoopBarriers::ProcessLoop(Loop *L, LPPassManager &)
-{
+LoopBarriers::ProcessLoop(Loop *L, LPPassManager &) {
   bool isBLoop = false;
   bool changed = false;
 
+  DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
+
   for (Loop::block_iterator i = L->block_begin(), e = L->block_end();
        i != e && !isBLoop; ++i) {
     for (BasicBlock::iterator j = (*i)->begin(), e = (*i)->end();
@@ -106,9 +96,9 @@ LoopBarriers::ProcessLoop(Loop *L, LPPassManager &)
         // Found a barrier in this loop:
         // 1) add a barrier in the loop header.
         // 2) add a barrier in the latches
-        
+
         // Add a barrier on the preheader to ensure all WIs reach
-        // the loop header with all the previous code already 
+        // the loop header with all the previous code already
         // executed.
         BasicBlock *preheader = L->getLoopPreheader();
         assert((preheader != NULL) && "Non-canonicalized loop found!\n");
@@ -156,11 +146,14 @@ LoopBarriers::ProcessLoop(Loop *L, LPPassManager &)
         // go trough all the latches.
         BasicBlock *Header = L->getHeader();
         typedef GraphTraits<Inverse<BasicBlock *> > InvBlockTraits;
-        InvBlockTraits::ChildIteratorType PI = InvBlockTraits::child_begin(Header);
-        InvBlockTraits::ChildIteratorType PE = InvBlockTraits::child_end(Header);
+        InvBlockTraits::ChildIteratorType PI =
+          InvBlockTraits::child_begin(Header);
+        InvBlockTraits::ChildIteratorType PE =
+          InvBlockTraits::child_end(Header);
+
         BasicBlock *Latch = NULL;
         for (; PI != PE; ++PI) {
-          InvBlockTraits::NodeType *N = *PI;
+          BasicBlock *N = *PI;
           if (L->contains(N)) {
             Latch = N;
             // Latch found in the loop, see if the barrier dominates it
@@ -178,12 +171,13 @@ LoopBarriers::ProcessLoop(Loop *L, LPPassManager &)
   }
 
   /* This is a loop without a barrier. Ensure we have a non-barrier
-     block as a preheader so we can replicate the loop as a whole. 
+     block as a preheader so we can replicate the loop as a whole.
 
      If the block has proper instructions after the barrier, it
      will be split in CanonicalizeBarriers. */
   BasicBlock *preheader = L->getLoopPreheader();
   assert((preheader != NULL) && "Non-canonicalized loop found!\n");
+
   TerminatorInst *t = preheader->getTerminator();
   Instruction *prev = NULL;
   if (&preheader->front() != t)
@@ -200,4 +194,3 @@ LoopBarriers::ProcessLoop(Loop *L, LPPassManager &)
 
   return changed;
 }
-
