@@ -28,21 +28,25 @@
 #include "llvm/IR/Function.h"
 #include "llvm/Pass.h"
 
+namespace llvm {
+class Loop;
+}
+
 namespace pocl {
   /**
    * Analyses the variables in the function to figure out if a variable
    * value is
    *
    * a) 'uniform', i.e., always same for all work-items in the *same work-group*
-   * b) 'varying', i.e., somehow dependent on the work-item id 
-   * 
+   * b) 'varying', i.e., somehow dependent on the work-item id
+   *
    * For safety, 'variable' is assumed, unless certain of a).
    *
-   * VAU is an "accumulating" pass; it gathers uniformity information of 
-   * instructions in a way that it should invalidate even though the CFG
-   * is modified. Thus, in case the semantics of the original information
-   * does not change, it is safe for passes to set this pass preserved even
-   * though new instructions are added or the CFG manipulated.
+   * VAU is an "accumulating" pass; it gathers uniformity information of
+   * instructions in a way that it needs not to be invalidated even though
+   * the CFG is modified. Thus, in case the semantics of the original
+   * information does not change, it is safe for passes to set this pass
+   * preserved even though new instructions are added or the CFG manipulated.
    */
   class VariableUniformityAnalysis : public llvm::FunctionPass {
   public:
@@ -60,6 +64,7 @@ namespace pocl {
 
     virtual bool shouldBePrivatized(llvm::Function *f, llvm::Value *val);
     virtual bool doFinalization(llvm::Module& M);
+    virtual void markInductionVariables(llvm::Function &F, llvm::Loop &L);
 
   private:
 
