@@ -97,6 +97,7 @@ namespace llvm {
     static StructType *get(LLVMContext &Context) {
       if (size_t_width == 64)
         {
+#ifdef LLVM_OLDER_THAN_5_0
           return StructType::get
             (TypeBuilder<types::i<32>, xcompile>::get(Context),
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
@@ -104,9 +105,24 @@ namespace llvm {
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
              NULL);
+#else
+          SmallVector<Type*, 8> Elements;
+          Elements.push_back(
+            TypeBuilder<types::i<32>, xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<64>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<64>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<64>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<64>[3], xcompile>::get(Context));
+          return StructType::get(Context, Elements);
+#endif
         }
       else if (size_t_width == 32)
         {
+#ifdef LLVM_OLDER_THAN_5_0
           return StructType::get
             (TypeBuilder<types::i<32>, xcompile>::get(Context),
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
@@ -114,6 +130,20 @@ namespace llvm {
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
              NULL);
+#else
+          SmallVector<Type*, 8> Elements;
+          Elements.push_back(
+            TypeBuilder<types::i<32>, xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<32>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<32>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<32>[3], xcompile>::get(Context));
+          Elements.push_back(
+            TypeBuilder<types::i<32>[3], xcompile>::get(Context));
+          return StructType::get(Context, Elements);
+#endif
         }
       else
         {
@@ -183,8 +213,6 @@ Workgroup::runOnModule(Module &M)
     if (!isKernelToProcess(*i)) continue;
     Function *L = createLauncher(M, &*i);
 
-    L->addFnAttr(Attribute::NoInline);
-
     privatizeContext(M, L);
 
     if (!currentPoclDevice->spmd) {
@@ -208,10 +236,15 @@ Workgroup::runOnModule(Module &M)
       }
   }
 
+#if LLVM_OLDER_THAN_5_0
   Function *barrier = cast<Function>
     (M.getOrInsertFunction(BARRIER_FUNCTION_NAME,
-                           Type::getVoidTy(M.getContext()),
-                           NULL));
+                           Type::getVoidTy(M.getContext()), NULL));
+#else
+  Function *barrier = cast<Function>
+    (M.getOrInsertFunction(BARRIER_FUNCTION_NAME,
+                           Type::getVoidTy(M.getContext())));
+#endif
   BasicBlock *bb = BasicBlock::Create(M.getContext(), "", barrier);
   ReturnInst::Create(M.getContext(), 0, bb);
 
@@ -293,7 +326,7 @@ createLauncher(Module &M, Function *F) {
 
   SmallVector<Value *, 8> arguments;
   Function::arg_iterator ai = L->arg_begin();
-  for (unsigned i = 0, e = F->getArgumentList().size(); i != e; ++i) {
+  for (unsigned i = 0, e = F->arg_size(); i != e; ++i) {
     arguments.push_back(&*ai);
     ++ai;
   }
@@ -345,7 +378,7 @@ createLauncher(Module &M, Function *F) {
   CallInst *c = builder.CreateCall(F, ArrayRef<Value*>(arguments));
   builder.CreateRetVoid();
 
-#ifdef LLVM_4_0
+#ifndef LLVM_OLDER_THAN_4_0
   // At least with LLVM 4.0, the runtime of AddAliasScopeMetadata of
   // llvm::InlineFunction explodes in case of kernels with restrict
   // metadata and a lot of lifetime markers. The issue produces at
@@ -361,10 +394,10 @@ createLauncher(Module &M, Function *F) {
       if (!llvm::isa<CallInst>(Instr)) continue;
       CallInst *CallInstr = dyn_cast<CallInst>(Instr);
       if (CallInstr->getCalledFunction() != nullptr &&
-          (CallInstr->getCalledFunction()->getName() ==
-           "llvm.lifetime.end" ||
-           CallInstr->getCalledFunction()->getName() ==
-           "llvm.lifetime.start")) {
+          (CallInstr->getCalledFunction()->getName().
+	   startswith("llvm.lifetime.end") ||
+           CallInstr->getCalledFunction()->getName().
+	   startswith("llvm.lifetime.start"))) {
         Calls.insert(CallInstr);
       }
     }
@@ -600,8 +633,6 @@ createWorkgroupFast(Module &M, Function *F)
     dyn_cast<Function>(M.getOrInsertFunction(funcName + "_workgroup_fast", ft));
   assert(workgroup != NULL);
 
-  workgroup->addFnAttr(Attribute::NoInline);
-
   builder.SetInsertPoint(BasicBlock::Create(M.getContext(), "", workgroup));
 
   Function::arg_iterator ai = workgroup->arg_begin();
