@@ -143,6 +143,8 @@ WorkitemLoops::runOnFunction(Function &F)
   contextArrays.clear();
   tempInstructionIds.clear();
 
+  releaseParallelRegions();
+
   return changed;
 }
 
@@ -251,19 +253,26 @@ WorkitemLoops::CreateLoopAround
 
   IRBuilder<> builder(forInitBB);
 
-  if (peeledFirst)
-    {
-      builder.CreateStore(builder.CreateLoad(localIdXFirstVar), localIdVar);
-      builder.CreateStore
-        (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdXFirstVar);
-    }
-  else
-    {
-      builder.CreateStore
-        (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdVar);
-    }
+  if (peeledFirst) {
+    builder.CreateStore(builder.CreateLoad(localIdXFirstVar), localIdVar);
+    builder.CreateStore
+      (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdXFirstVar);
 
-  builder.CreateBr(loopBodyEntryBB);
+    if (WGDynamicLocalSize) {
+      llvm::Value *cmpResult;
+      cmpResult = builder.CreateICmpULT(builder.CreateLoad(localIdVar),
+                                        builder.CreateLoad(DynamicLocalSize));
+
+      builder.CreateCondBr(cmpResult, loopBodyEntryBB, loopEndBB);
+    } else {
+      builder.CreateBr(loopBodyEntryBB);
+    }
+  } else {
+    builder.CreateStore
+      (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdVar);
+
+    builder.CreateBr(loopBodyEntryBB);
+  }
 
   exitBB->getTerminator()->replaceUsesOfWith(oldExit, forCondBB);
   if (addIncBlock)
@@ -334,6 +343,20 @@ WorkitemLoops::RegionOfBlock(llvm::BasicBlock *bb)
   return NULL;
 }
 
+void WorkitemLoops::releaseParallelRegions() {
+  if (original_parallel_regions) {
+    for (auto i = original_parallel_regions->begin(),
+              e = original_parallel_regions->end();
+              i != e; ++i) {
+      ParallelRegion *p = *i;
+      delete p;
+    }
+
+    delete original_parallel_regions;
+    original_parallel_regions = nullptr;
+  }
+}
+
 bool
 WorkitemLoops::ProcessFunction(Function &F)
 {
@@ -351,6 +374,8 @@ WorkitemLoops::ProcessFunction(Function &F)
       return true;
     }
 
+  releaseParallelRegions();
+
 #ifdef LLVM_OLDER_THAN_3_7
   original_parallel_regions = K->getParallelRegions(LI);
 #else
@@ -916,13 +941,50 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
     }
   else
     {
-      /* 3D context array. */
-      llvm::Type *contextArrayType =
-        ArrayType::get(
-          ArrayType::get(
-            ArrayType::get(
-                           elementType, WGLocalSizeX),
-            WGLocalSizeY), WGLocalSizeZ);
+      /* 3D context array. In case the elementType itself is an array or struct,
+       * we must take into account it could be alloca-ed with alignment and loads
+       * or stores might use vectorized instructions expecting proper alignment.
+       * Because of that, we cannot simply allocate x*y*z*(size), we must
+       * enlarge the array type to fit the alignment. */
+      Type *allocType = elementType;
+      AllocaInst *allocaInst = dyn_cast<AllocaInst>(instruction);
+      if (allocaInst) {
+        unsigned alignment = allocaInst->getAlignment();
+
+        const DataLayout &dataLayout = M->getDataLayout();
+        uint64_t storeSize =
+          dataLayout.getTypeStoreSize(allocaInst->getAllocatedType());
+
+        if ((alignment > 1) && (storeSize & (alignment - 1))) {
+          uint64_t alignedSize = (storeSize & (~(alignment - 1))) + alignment;
+#ifdef DEBUG_WORK_ITEM_LOOPS
+        std::cerr << "### unaligned type found: aligning " << storeSize
+                  << " to " << alignedSize << "\n";
+#endif
+          if (isa<ArrayType>(elementType)) {
+            allocType =
+              ArrayType::get(elementType->getArrayElementType(), alignedSize);
+          } else if (isa<StructType>(elementType)) {
+            StructType *old_struct = dyn_cast<StructType>(elementType);
+
+            unsigned required_bytes = alignedSize - storeSize;
+            ArrayType *structPadding = ArrayType::get(
+                Type::getInt8Ty(M->getContext()), required_bytes);
+            std::vector<Type *> ary;
+            for (unsigned j = 0; j < old_struct->getNumElements(); j++)
+              ary.push_back(old_struct->getElementType(j));
+            ary.push_back(structPadding);
+            const ArrayRef<Type *> new_el(ary);
+            allocType = StructType::get(old_struct->getContext(), new_el,
+                                        old_struct->isPacked());
+            unsigned newStoreSize = dataLayout.getTypeStoreSize(allocType);
+            assert(newStoreSize == alignedSize);
+          }
+        }
+      }
+      llvm::Type *contextArrayType = ArrayType::get(
+          ArrayType::get(ArrayType::get(allocType, WGLocalSizeX), WGLocalSizeY),
+          WGLocalSizeZ);
 
       /* Allocate the context data array for the variable. */
       Alloca = builder.CreateAlloca(contextArrayType, 0, varName);
