@@ -305,6 +305,60 @@ stringref_cmp(llvm::StringRef a, llvm::StringRef b)
     return a.str() < b.str();
 }
 
+static void shared_copy(llvm::Module *program, const llvm::Module *lib,
+                        std::string &log, ValueToValueMapTy &vvm) {
+
+  llvm::Module::const_global_iterator gi,ge;
+
+  // copy any aliases to program
+  DB_PRINT("cloning the aliases:\n");
+  llvm::Module::const_alias_iterator ai, ae;
+  for (ai = lib->alias_begin(), ae = lib->alias_end(); ai != ae; ai++) {
+    DB_PRINT(" %s\n", ai->getName().data());
+    GlobalAlias *GA =
+#ifndef LLVM_3_7
+      GlobalAlias::create(
+        ai->getType(), ai->getType()->getAddressSpace(), ai->getLinkage(),
+        ai->getName(), NULL, program);
+#else
+    GlobalAlias::create(
+        ai->getType(), ai->getLinkage(), ai->getName(), NULL, program);
+#endif
+
+    GA->copyAttributesFrom(&*ai);
+    vvm[&*ai]=GA;
+  }
+
+  // initialize the globals that were copied
+  for (gi=lib->global_begin(), ge=lib->global_end();
+       gi != ge;
+       gi++) {
+      GlobalVariable *GV=cast<GlobalVariable>(vvm[&*gi]);
+      if (gi->hasInitializer())
+        GV->setInitializer(MapValue(gi->getInitializer(), vvm));
+  }
+
+  // copy metadata
+  DB_PRINT("cloning metadata:\n");
+  llvm::Module::const_named_metadata_iterator mi,me;
+  for (mi=lib->named_metadata_begin(), me=lib->named_metadata_end();
+       mi != me; mi++) {
+      const NamedMDNode &NMD=*mi;
+      /* This causes problems, because multiple wchar_size */
+      if (NMD.getName() == StringRef("llvm.module.flags"))
+        continue;
+      /* This causes problems with NVidia,
+       * and is regenerated by pocl-ptx-gen anyway */
+      if (NMD.getName() == StringRef("nvvm.annotations"))
+        continue;
+      DB_PRINT(" %s:\n", NMD.getName().data());
+      NamedMDNode *NewNMD=program->getOrInsertNamedMetadata(NMD.getName());
+      for (unsigned i=0, e=NMD.getNumOperands(); i != e; ++i)
+        NewNMD->addOperand(MapMetadata(NMD.getOperand(i), vvm));
+  }
+
+}
+
 int link(llvm::Module *program, const llvm::Module *lib, std::string &log) {
   assert(program);
   assert(lib);
@@ -384,45 +438,35 @@ int link(llvm::Module *program, const llvm::Module *lib, std::string &log) {
   if (!found_all_undefined)
     return 1;
 
-  // copy any aliases to program
-  DB_PRINT("cloning the aliases:\n");
-  llvm::Module::const_alias_iterator ai, ae;
-  for (ai = lib->alias_begin(), ae = lib->alias_end(); ai != ae; ai++) {
-    DB_PRINT(" %s\n", ai->getName().data());
-    GlobalAlias *GA =
-#ifndef LLVM_3_7
-      GlobalAlias::create(
-	ai->getType(), ai->getType()->getAddressSpace(), ai->getLinkage(),
-	ai->getName(), NULL, program);
-#else
-    GlobalAlias::create(
-        ai->getType(), ai->getLinkage(), ai->getName(), NULL, program);
-#endif
+  shared_copy(program, lib, log, vvm);
 
-    GA->copyAttributesFrom(&*ai);
-    vvm[&*ai]=GA;
+  return 0;
+}
+
+int copyKernelFromBitcode(const char* name, llvm::Module *parallel_bc,
+                          const llvm::Module *program) {
+  ValueToValueMapTy vvm;
+
+  // Copy all the globals from lib to program.
+  // It probably is faster to just copy them all, than to inspect
+  // both program and lib to find which actually are used.
+  DB_PRINT("cloning the global variables:\n");
+  llvm::Module::const_global_iterator gi,ge;
+  for (gi=program->global_begin(), ge=program->global_end(); gi != ge; gi++) {
+    DB_PRINT(" %s\n", gi->getName().data());
+    GlobalVariable *GV = new GlobalVariable(
+      *parallel_bc, gi->getType()->getElementType(), gi->isConstant(),
+      gi->getLinkage(), (Constant*)0, gi->getName(), (GlobalVariable*)0,
+      gi->getThreadLocalMode(), gi->getType()->getAddressSpace());
+    GV->copyAttributesFrom(&*gi);
+    vvm[&*gi]=GV;
   }
 
-  // initialize the globals that were copied
-  for (gi=lib->global_begin(), ge=lib->global_end();
-       gi != ge;
-       gi++) {
-      GlobalVariable *GV=cast<GlobalVariable>(vvm[&*gi]);
-      if (gi->hasInitializer())
-        GV->setInitializer(MapValue(gi->getInitializer(), vvm));
-  }
+  const StringRef kernel_name(name);
+  copy_func_callgraph(kernel_name, program, parallel_bc, vvm);
 
-  // copy metadata
-  DB_PRINT("cloning metadata:\n");
-  llvm::Module::const_named_metadata_iterator mi,me;
-  for (mi=lib->named_metadata_begin(), me=lib->named_metadata_end();
-       mi != me; mi++) {
-      const NamedMDNode &NMD=*mi;
-      DB_PRINT(" %s:\n", NMD.getName().data());
-      NamedMDNode *NewNMD=program->getOrInsertNamedMetadata(NMD.getName());
-      for (unsigned i=0, e=NMD.getNumOperands(); i != e; ++i)
-        NewNMD->addOperand(MapMetadata(NMD.getOperand(i), vvm));
-  }
+  std::string log;
+  shared_copy(parallel_bc, program, log, vvm);
 
   return 0;
 }
