@@ -23,8 +23,10 @@
 */
 
 // Enable OpenCL C++ exceptions
-#define __CL_ENABLE_EXCEPTIONS
-#include <CL/cl.hpp>
+#define CL_HPP_ENABLE_EXCEPTIONS
+#define CL_HPP_MINIMUM_OPENCL_VERSION 120
+#define CL_HPP_TARGET_OPENCL_VERSION 120
+#include <CL/cl2.hpp>
 
 #include <cstdio>
 #include <cstdlib>
@@ -61,7 +63,7 @@ main(void)
     int a = 3;
 
     for (int i = 0; i < BUFFER_SIZE; i++) {
-        A[i] = i;
+        A[i] = (float)i;
     }
 
     for (int i = 0; i < WORK_ITEMS; i++) {
@@ -83,7 +85,7 @@ main(void)
         std::vector<cl::Device> devices = context.getInfo<CL_CONTEXT_DEVICES>();
 
         // Create and program from source
-        cl::Program::Sources sources(1, std::make_pair(kernelSourceCode, 0));
+        cl::Program::Sources sources({kernelSourceCode});
         cl::Program program(context, sources);
 
         // Build program
@@ -141,20 +143,21 @@ main(void)
                 ok = false;
             }
         }
-        if (ok) 
-          return EXIT_SUCCESS;
-        else
-          return EXIT_FAILURE;
 
         // Finally release our hold on accessing the memory
         queue.enqueueUnmapMemObject(
             cBuffer,
             (void *) output);
- 
+
         // There is no need to perform a finish on the final unmap
         // or release any objects as this all happens implicitly with
         // the C++ Wrapper API.
-    } 
+
+        if (ok)
+          return EXIT_SUCCESS;
+        else
+          return EXIT_FAILURE;
+    }
     catch (cl::Error err) {
          std::cerr
              << "ERROR: "
