@@ -26,8 +26,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "poclu.h"
-#include "pocl_tests.h"
 #include "config.h"
+#include "pocl.h"
 
 char kernelSourceCode[] =
 "constant sampler_t samp =  CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_NONE | CLK_FILTER_NEAREST;\n"
@@ -77,30 +77,36 @@ int test_program(cl_program program, int is_spir) {
   err = clGetKernelArgInfo(test_kernel, 0, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_CONSTANT) && "arg of test_kernel is not CONSTANT addr space");
+  printf ("adr: %x\n", kernel_arg.address);
+  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_CONSTANT) &&
+	      "arg of test_kernel is not CONSTANT addr space");
 
   err = clGetKernelArgInfo(test_kernel, 1, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_GLOBAL) && "arg of test_kernel is not GLOBAL addr space");
+  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_GLOBAL) &&
+	      "arg of test_kernel is not GLOBAL addr space");
 
 
   err = clGetKernelArgInfo(test_kernel, 2, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_GLOBAL) && "arg of test_kernel is not GLOBAL addr space");
+  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_GLOBAL) &&
+	      "arg of test_kernel is not GLOBAL addr space");
 
 
   err = clGetKernelArgInfo(test_kernel, 3, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_PRIVATE) && "arg of test_kernel is not PRIVATE addr space");
+  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_PRIVATE) &&
+	      "arg of test_kernel is not PRIVATE addr space");
 
 
   err = clGetKernelArgInfo(test_kernel, 4, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_LOCAL) && "arg of test_kernel is not LOCAL addr space");
+  TEST_ASSERT((kernel_arg.address==CL_KERNEL_ARG_ADDRESS_LOCAL) &&
+	      "arg of test_kernel is not LOCAL addr space");
 
   /* ACCESS QUALIFIER tests */
   // constant char* msg, global volatile float* in, global float* out, const float j, local int* c
@@ -109,7 +115,8 @@ int test_program(cl_program program, int is_spir) {
     err = clGetKernelArgInfo(test_kernel, i, CL_KERNEL_ARG_ACCESS_QUALIFIER,
                               BUF_LEN, &kernel_arg.access, &retsize);
     CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-    TEST_ASSERT((kernel_arg.access==CL_KERNEL_ARG_ACCESS_NONE) && "arg of test_kernel is not NONE access");
+    TEST_ASSERT((kernel_arg.access==CL_KERNEL_ARG_ACCESS_NONE) &&
+		"arg of test_kernel is not NONE access");
   }
 
   /* TYPE NAME tests */
@@ -118,27 +125,34 @@ int test_program(cl_program program, int is_spir) {
   err = clGetKernelArgInfo(test_kernel, 0, CL_KERNEL_ARG_TYPE_NAME,
                             BUF_LEN, &kernel_arg.string, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((retsize==6) && " arg type name size of test_kernel doesnt fit");
-  TEST_ASSERT((strncmp(kernel_arg.string, "char*", 6)==0) && " arg type name of test_kernel doesnt compare");
-
+  TEST_ASSERT ((retsize == 6)
+               && " arg type name size of test_kernel doesn't fit");
+  TEST_ASSERT ((strncmp (kernel_arg.string, "char*", 6) == 0)
+               && " arg type name of test_kernel doesn't compare");
 
   err = clGetKernelArgInfo(test_kernel, 1, CL_KERNEL_ARG_TYPE_NAME,
                             BUF_LEN, &kernel_arg.string, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((retsize==7) && " arg type name size of test_kernel doesnt fit");
-  TEST_ASSERT((strncmp(kernel_arg.string, "float*", 7)==0) && " arg type name of test_kernel doesnt compare");
+  TEST_ASSERT ((retsize == 7)
+               && " arg type name size of test_kernel doesn't fit");
+  TEST_ASSERT ((strncmp (kernel_arg.string, "float*", 7) == 0)
+               && " arg type name of test_kernel doesn't compare");
 
   err = clGetKernelArgInfo(test_kernel, 3, CL_KERNEL_ARG_TYPE_NAME,
                             BUF_LEN, &kernel_arg.string, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((retsize==6) && " arg type name size of test_kernel doesnt fit");
-  TEST_ASSERT((strncmp(kernel_arg.string, "float", 6)==0) && " arg type name of test_kernel doesnt compare");
+  TEST_ASSERT ((retsize == 6)
+               && " arg type name size of test_kernel doesn't fit");
+  TEST_ASSERT ((strncmp (kernel_arg.string, "float", 6) == 0)
+               && " arg type name of test_kernel doesn't compare");
 
   err = clGetKernelArgInfo(test_kernel, 4, CL_KERNEL_ARG_TYPE_NAME,
                             BUF_LEN, &kernel_arg.string, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((retsize==5) && " arg type name size of test_kernel doesnt fit");
-  TEST_ASSERT((strncmp(kernel_arg.string, "int*", 5)==0) && " arg type name of test_kernel doesnt compare");
+  TEST_ASSERT ((retsize == 5)
+               && " arg type name size of test_kernel doesn't fit");
+  TEST_ASSERT ((strncmp (kernel_arg.string, "int*", 5) == 0)
+               && " arg type name of test_kernel doesn't compare");
 
   /* TYPE QUALIFIER tests */
   // constant char* msg, global volatile float* in, global float* out, const float j, local int* c
@@ -146,22 +160,57 @@ int test_program(cl_program program, int is_spir) {
   err = clGetKernelArgInfo(test_kernel, 0, CL_KERNEL_ARG_TYPE_QUALIFIER,
                             BUF_LEN, &kernel_arg.type, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_CONST) && "type qualifier of arg of test_kernel is not CONST");
+  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_CONST) &&
+	      "type qualifier of arg of test_kernel is not CONST");
 
   err = clGetKernelArgInfo(test_kernel, 1, CL_KERNEL_ARG_TYPE_QUALIFIER,
                             BUF_LEN, &kernel_arg.type, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_VOLATILE) && "type qualifier of arg of test_kernel is not VOLATILE");
+  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_VOLATILE) &&
+	      "type qualifier of arg of test_kernel is not VOLATILE");
 
   err = clGetKernelArgInfo(test_kernel, 2, CL_KERNEL_ARG_TYPE_QUALIFIER,
                             BUF_LEN, &kernel_arg.type, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_NONE) && "type qualifier of arg of test_kernel is not NONE");
+  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_NONE) &&
+	      "type qualifier of arg of test_kernel is not NONE");
 
-  err = clGetKernelArgInfo(test_kernel, 3, CL_KERNEL_ARG_TYPE_QUALIFIER,
-                            BUF_LEN, &kernel_arg.type, &retsize);
-  CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((kernel_arg.type==CL_KERNEL_ARG_TYPE_CONST) && "type qualifier of arg of test_kernel is not CONST");
+#ifndef LLVM_OLDER_THAN_5_0
+  if (!is_spir) {
+
+    /* Clang versions before 5 added the const MD also for non-pointer
+       types even though OpenCL specs mandates to put them only in
+       pointer args. This was fixed in Clang r299192 (see below).
+       TODO: update the SPIRs with 5.0+ Clang to drop the metadata. */
+
+    /*
+      r299192 | echuraev | 2017-03-31 13:14:52 +0300 (Fri, 31 Mar 2017) | 26 lines
+
+      [OpenCL] Do not generate "kernel_arg_type_qual" metadata for non-pointer args
+
+      Summary:
+      "kernel_arg_type_qual" metadata should contain const/volatile/restrict
+      tags only for pointer types to match the corresponding requirement of
+      the OpenCL specification.
+
+      OpenCL 2.0 spec 5.9.3 Kernel Object Queries:
+
+      CL_KERNEL_ARG_TYPE_VOLATILE is returned if the argument is a pointer
+      and the referenced type is declared with the volatile qualifier.
+      [...]
+      Similarly, CL_KERNEL_ARG_TYPE_CONST is returned if the argument is a
+      pointer and the referenced type is declared with the restrict or const
+      qualifier.
+      [...]
+      CL_KERNEL_ARG_TYPE_RESTRICT will be returned if the pointer type is
+      marked restrict.
+    */
+    err = clGetKernelArgInfo(test_kernel, 3, CL_KERNEL_ARG_TYPE_QUALIFIER,
+                             BUF_LEN, &kernel_arg.type, &retsize);
+    CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
+    TEST_ASSERT((kernel_arg.type == CL_KERNEL_ARG_TYPE_NONE));
+  }
+#endif
 
   /* NAME tests */
   // constant char* msg, global volatile float* in, global float* out, const float j, local int* c
@@ -172,8 +221,10 @@ int test_program(cl_program program, int is_spir) {
     printf("arg name not available (this is normal for SPIR)\n");
   } else {
     CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-    TEST_ASSERT((retsize==4) && " arg name size of test_kernel doesnt fit");
-    TEST_ASSERT((strncmp(kernel_arg.string, "msg", 4)==0) && " arg name of test_kernel doesnt compare");
+    TEST_ASSERT ((retsize == 4)
+                 && " arg name size of test_kernel doesn't fit");
+    TEST_ASSERT ((strncmp (kernel_arg.string, "msg", 4) == 0)
+                 && " arg name of test_kernel doesn't compare");
   }
 
   err = clGetKernelArgInfo(test_kernel, 1, CL_KERNEL_ARG_NAME,
@@ -182,8 +233,10 @@ int test_program(cl_program program, int is_spir) {
     printf("arg name not available (this is normal for SPIR)\n");
   } else {
     CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-    TEST_ASSERT((retsize==3) && " arg name size of test_kernel doesnt fit");
-    TEST_ASSERT((strncmp(kernel_arg.string, "in", 3)==0) && " arg name of test_kernel doesnt compare");
+    TEST_ASSERT ((retsize == 3)
+                 && " arg name size of test_kernel doesn't fit");
+    TEST_ASSERT ((strncmp (kernel_arg.string, "in", 3) == 0)
+                 && " arg name of test_kernel doesn't compare");
   }
 
   err = clGetKernelArgInfo(test_kernel, 2, CL_KERNEL_ARG_NAME,
@@ -192,8 +245,10 @@ int test_program(cl_program program, int is_spir) {
     printf("arg name not available (this is normal for SPIR)\n");
   } else {
     CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-    TEST_ASSERT((retsize==4) && " arg name size of test_kernel doesnt fit");
-    TEST_ASSERT((strncmp(kernel_arg.string, "out", 4)==0) && " arg name of test_kernel doesnt compare");
+    TEST_ASSERT ((retsize == 4)
+                 && " arg name size of test_kernel doesn't fit");
+    TEST_ASSERT ((strncmp (kernel_arg.string, "out", 4) == 0)
+                 && " arg name of test_kernel doesn't compare");
   }
 
   err = clGetKernelArgInfo(test_kernel, 3, CL_KERNEL_ARG_NAME,
@@ -202,8 +257,10 @@ int test_program(cl_program program, int is_spir) {
     printf("arg name not available (this is normal for SPIR)\n");
   } else {
     CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-    TEST_ASSERT((retsize==2) && " arg name size of test_kernel doesnt fit");
-    TEST_ASSERT((strncmp(kernel_arg.string, "j", 2)==0) && " arg name of test_kernel doesnt compare");
+    TEST_ASSERT ((retsize == 2)
+                 && " arg name size of test_kernel doesn't fit");
+    TEST_ASSERT ((strncmp (kernel_arg.string, "j", 2) == 0)
+                 && " arg name of test_kernel doesn't compare");
   }
 
   /* ACCESS QUALIFIER tests for test_kernel2 */
@@ -228,8 +285,10 @@ int test_program(cl_program program, int is_spir) {
   err = clGetKernelArgInfo(test_kernel2, 0, CL_KERNEL_ARG_TYPE_NAME,
                             BUF_LEN, &kernel_arg.string, &retsize);
   CHECK_OPENCL_ERROR_IN("clGetKernelArgInfo");
-  TEST_ASSERT((retsize==12) && " arg type name size of test_kernel doesnt fit");
-  TEST_ASSERT((strncmp(kernel_arg.string, "tasty_float", 12)==0) && " arg type name of test_kernel2 doesnt compare");
+  TEST_ASSERT ((retsize == 12)
+               && " arg type name size of test_kernel doesn't fit");
+  TEST_ASSERT ((strncmp (kernel_arg.string, "tasty_float", 12) == 0)
+               && " arg type name of test_kernel2 doesn't compare");
 
   err = clReleaseKernel(test_kernel);
   CHECK_OPENCL_ERROR_IN("clReleaseKernel");
@@ -239,6 +298,7 @@ int test_program(cl_program program, int is_spir) {
 
 }
 
+/*
 int test_program_nometa(cl_program program) {
 
   cl_int err;
@@ -255,8 +315,6 @@ int test_program_nometa(cl_program program) {
   CHECK_OPENCL_ERROR_IN("clCreateKernel");
 
 
-  /* ADDR SPACE QUALIFIER tests */
-  // constant char* msg, global volatile float* in, global float* out, const float j, local int* c
   err = clGetKernelArgInfo(test_kernel, 0, CL_KERNEL_ARG_ADDRESS_QUALIFIER,
                             BUF_LEN, &kernel_arg.address, &retsize);
   TEST_ASSERT(err == CL_KERNEL_ARG_INFO_NOT_AVAILABLE);
@@ -273,41 +331,33 @@ int test_program_nometa(cl_program program) {
                             BUF_LEN, &kernel_arg.string, &retsize);
   TEST_ASSERT(err == CL_KERNEL_ARG_INFO_NOT_AVAILABLE);
 
+  err = clReleaseKernel (test_kernel);
+  CHECK_OPENCL_ERROR_IN ("clReleaseKernel");
+
   return EXIT_SUCCESS;
 }
+*/
 
-int spir_program(const char * filename, cl_context ctx, cl_device_id did, cl_program* program) {
+int spir_program(char * filename, cl_context ctx, cl_device_id did, cl_program* program) {
   cl_int err;
+
   size_t program_size;
   char* program_buffer;
-  FILE *spir_bitcode;
 
-  printf("opening File: %s\n", filename);
-  spir_bitcode = fopen(filename, "r");
-  TEST_ASSERT(spir_bitcode && "couldn't find spir bitcode file for this test");
-
-  fseek (spir_bitcode, 0, SEEK_END);
-  program_size = ftell (spir_bitcode);
-  fseek (spir_bitcode, 0, SEEK_SET);
+  program_buffer = poclu_read_binfile(filename, &program_size);
   TEST_ASSERT(program_size > 2000);
+  TEST_ASSERT(program_buffer != NULL);
   printf("program size: %zi\n", program_size);
 
-  program_buffer = (char *) malloc (program_size +1 );
-  TEST_ASSERT(program_buffer);
-
-  TEST_ASSERT(fread(program_buffer, program_size, 1, spir_bitcode) == 1);
-
-  TEST_ASSERT(fclose(spir_bitcode) == 0);
-
-
   *program = clCreateProgramWithBinary (ctx, 1, &did, &program_size,
                                        (const unsigned char**)&program_buffer,
                                         NULL, &err);
   CHECK_OPENCL_ERROR_IN("clCreateProgramWithBinary");
   TEST_ASSERT(program);
 
-  err = clBuildProgram (*program, 1, &did, NULL, NULL, NULL);
-  CHECK_OPENCL_ERROR_IN("clBuildProgram");
+  CHECK_CL_ERROR(clBuildProgram (*program, 1, &did, NULL, NULL, NULL));
+
+  free (program_buffer);
 
   return EXIT_SUCCESS;
 }
@@ -325,7 +375,6 @@ int main()
 
   cl_program program = NULL;
 
-
   poclu_get_any_device(&ctx, &did, &queue);
   TEST_ASSERT(ctx);
   TEST_ASSERT(did);
@@ -346,28 +395,43 @@ int main()
   CHECK_OPENCL_ERROR_IN("clBuildProgram");
 
   printf("\nNON-SPIR\n");
-  if (test_program(program, 0) != EXIT_SUCCESS) return EXIT_FAILURE;
+  TEST_ASSERT(test_program(program, 0) == EXIT_SUCCESS);
 
-  err = clReleaseProgram(program);
-  CHECK_OPENCL_ERROR_IN("clReleaseProgram");
+  CHECK_CL_ERROR(clReleaseProgram(program));
+
+  char extensions[1024];
+  err = clGetDeviceInfo(did, CL_DEVICE_EXTENSIONS, 1024, extensions, NULL);
+  CHECK_OPENCL_ERROR_IN("clGetDeviceInfo");
+  if (strstr(extensions, "cl_khr_spir") == NULL)
+    {
+      printf ("SPIR not supported, skipping SPIR arg info tests\n");
+      goto FINISH;
+    }
 
   /* SPIR program */
 
   printf("\nSPIR with metadata\n");
-  if(spir_program(SPIR_FILE(POCL_DEVICE_ADDRESS_BITS, "_meta"), ctx, did, &program) != EXIT_SUCCESS) return EXIT_FAILURE;
+  TEST_ASSERT(spir_program(SPIR_FILE(POCL_DEVICE_ADDRESS_BITS, "_meta"), ctx, did, &program) == EXIT_SUCCESS);
 
-  if (test_program(program, 1) != EXIT_SUCCESS) return EXIT_FAILURE;
+  TEST_ASSERT(test_program(program, 1) == EXIT_SUCCESS);
 
-  err = clReleaseProgram(program);
-  CHECK_OPENCL_ERROR_IN("clReleaseProgram");
+  CHECK_CL_ERROR(clReleaseProgram(program));
 
+  /* SPIR program without metadata - currently disabled since Clang seems to
+   * always generate metadata. */
+/*
   printf("\nSPIR WITHOUT metadata\n");
-  if(spir_program(SPIR_FILE(POCL_DEVICE_ADDRESS_BITS, "_nometa"), ctx, did, &program) != EXIT_SUCCESS) return EXIT_FAILURE;
+  TEST_ASSERT(spir_program(SPIR_FILE(POCL_DEVICE_ADDRESS_BITS, "_nometa"), ctx, did, &program) == EXIT_SUCCESS);
 
-  if (test_program_nometa(program) != EXIT_SUCCESS) return EXIT_FAILURE;
+  TEST_ASSERT(test_program_nometa(program) == EXIT_SUCCESS);
 
-  err = clReleaseProgram(program);
-  CHECK_OPENCL_ERROR_IN("clReleaseProgram");
+  CHECK_CL_ERROR(clReleaseProgram(program));
+*/
+
+FINISH:
+  CHECK_CL_ERROR (clReleaseCommandQueue (queue));
+  CHECK_CL_ERROR (clReleaseContext (ctx));
+  CHECK_CL_ERROR (clUnloadCompiler ());
 
   printf("\nOK\n");
   return EXIT_SUCCESS;
