@@ -44,7 +44,7 @@ int main(int argc, char **argv)
   CHECK_OPENCL_ERROR_IN("clCreateBuffer");
 
   cl_image_format img_fmt = {
-    .image_channel_order = CL_R,
+    .image_channel_order = CL_RGBA,
     .image_channel_data_type = CL_UNSIGNED_INT32 };
   cl_image_desc img_dsc = {
     .image_type = CL_MEM_OBJECT_IMAGE2D,
@@ -208,6 +208,13 @@ int main(int argc, char **argv)
     TEST_ASSERT(map_event == initial_value);
   }
 
+  clFinish (queue);
+  clReleaseMemObject (img);
+  clReleaseMemObject (buf);
+
+  CHECK_CL_ERROR (clReleaseCommandQueue (queue));
+  CHECK_CL_ERROR (clReleaseContext (ctx));
+
   return EXIT_SUCCESS;
 
 }
