@@ -27,6 +27,10 @@
 #include <CL/opencl.h>
 #include "poclu.h"
 
+#ifdef _MSC_VER
+#  include "vccompat.hpp"
+#endif
+
 /**
  * The test kernels are assumed to:
  *
@@ -88,7 +92,7 @@ main (int argc, char **argv)
     return -1; 
 
   clGetContextInfo(context, CL_CONTEXT_DEVICES, 0, NULL, &cb); 
-  devices = (cl_device_id *) malloc(cb); 
+  devices = (cl_device_id *) malloc(cb);
   clGetContextInfo(context, CL_CONTEXT_DEVICES, cb, devices, NULL); 
  
   cmd_queue = clCreateCommandQueue(context, devices[0], 0, NULL); 
